#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#!/usr/bin/env python
# encoding: utf-8

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

# Mon 14 Sep 13:15:34 CEST 2015


from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.auth.models import User
from django.db.models import Q

from .models import Attestation


def view(request, number):

    # Retrieve the attestation
    attestation = get_object_or_404(Attestation, number=number)
    (has_access, _) = attestation.experiment.accessibility_for(request.user)

    # Render the page
    return render_to_response('attestations/view.html',
            dict(
                attestation=attestation,
                has_access=has_access,
                owner=(attestation.experiment.author == request.user),
                ),
            context_instance=RequestContext(request))


def ls(request, author_name):
    '''List all accessible attestations to the request user'''

    if not author_name: return public_ls(request)

    # check that the user exists on the system
    author = get_object_or_404(User, username=author_name)

    if request.user == author:
        # list all of the users attestations
        objects = Attestation.objects.filter(Q(experiment__author=author) | \
            Q(locked=False))

    else:
        # list all unlocked (public) attestations from a given user
        objects = Attestation.objects.filter(experiment__author=author,
                locked=False)

    objects = objects.order_by('-locked', 'expiration_date')

    return render_to_response('attestations/list.html',
            dict(
                objects=objects,
                author=author,
                owner=(request.user==author),
                ),
            context_instance=RequestContext(request),
            )


def public_ls(request):
    '''List all accessible attestations to any user'''

    # orders so that recent objects are displayed first
    objects = Attestation.objects.filter(locked=False).order_by('-publication_date')

    return render_to_response('attestations/list.html',
            dict(
                objects=objects,
                author=request.user, #anonymous
                owner=False,
                ),
            context_instance=RequestContext(request),
            )
