#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('team', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Environment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sharing', models.CharField(default=b'P', max_length=1, choices=[(b'P', b'Private'), (b'S', b'Shared'), (b'A', b'Public'), (b'U', b'Usable')])),
                ('name', models.CharField(help_text=b'The name for this object (space-like characters will be automatically replaced by dashes)', max_length=200)),
                ('version', models.CharField(help_text=b'Free-style version for this environment (normally read from the Worker/Scheduler available environments)', max_length=20)),
                ('short_description', models.CharField(default=b'', help_text=b'Describe the object succinctly (try to keep it under 80 characters)', max_length=100, blank=True)),
                ('description', models.TextField(default=b'', help_text=b'Describe the object thoroughly using <a href="http://docutils.sourceforge.net/rst.html">reStructuredText mark-up</a><br/><i class="fa fa-thumbs-up"></i> The ruler at 80 columns indicate suggested <a href="https://en.wikipedia.org/wiki/POSIX">POSIX line breaks</a> (for readability).<br/><i class="fa fa-thumbs-up"></i> The editor will automatically enlarge to accomodate the entirety of your input<br/><i class="fa fa-thumbs-up"></i> Use <a href="http://codemirror.net/doc/manual.html#commands">keyboard shortcuts</a> for search/replace and faster editing. For example, use Ctrl-F (PC) or Cmd-F (Mac) to search through this box', blank=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name=b'Creation date')),
                ('active', models.BooleanField(default=True, help_text=b'If this environment can be used in experiments')),
                ('previous_version', models.ForeignKey(related_name='next_versions', blank=True, to='backend.Environment', null=True)),
                ('shared_with', models.ManyToManyField(related_name='shared_environments', to=settings.AUTH_USER_MODEL, blank=True)),
                ('shared_with_team', models.ManyToManyField(related_name='shared_environments', to='team.Team', blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Queue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(help_text=b'The name for this object (space-like characters will be automatically replaced by dashes)', max_length=100)),
                ('memory_limit', models.IntegerField(help_text=b'In megabytes')),
                ('time_limit', models.IntegerField(help_text=b'In minutes')),
                ('nb_cores_per_slot', models.IntegerField()),
                ('max_slots_per_user', models.IntegerField()),
                ('environments', models.ManyToManyField(related_name='queues', to='backend.Environment')),
            ],
            options={
                'permissions': [['can_access', 'Can access queue']],
            },
        ),
        migrations.CreateModel(
            name='QueueWorkerSlot',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('nb_slots', models.IntegerField(help_text=b'Number of processing slots to dedicate in this worker for a given queue', verbose_name=b'Number of slots')),
                ('queue', models.ForeignKey(related_name='slots', to='backend.Queue')),
            ],
        ),
        migrations.CreateModel(
            name='Worker',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(help_text=b'The name for this object (space-like characters will be automatically replaced by dashes)', unique=True, max_length=100)),
                ('active', models.BooleanField(default=True, help_text='If this worker is usable presently')),
                ('nb_cores', models.IntegerField()),
            ],
        ),
        migrations.AddField(
            model_name='queueworkerslot',
            name='worker',
            field=models.ForeignKey(related_name='slots', to='backend.Worker'),
        ),
        migrations.AlterUniqueTogether(
            name='queueworkerslot',
            unique_together=set([('queue', 'worker', 'nb_slots')]),
        ),
        migrations.AlterUniqueTogether(
            name='environment',
            unique_together=set([('name', 'version')]),
        ),
    ]
