#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

'''Utilities for summarizing scheduler state'''


import os
import logging
logger = logging.getLogger(__name__)

import psutil

from django.conf import settings

from .models import Job, JobSplit

from ..experiments.models import Experiment


def cache():
    '''Returns a current cache state'''

    if not os.path.exists(settings.CACHE_ROOT):
        os.makedirs(settings.CACHE_ROOT)
        mesg = "Created cache directory (%s)" % settings.CACHE_ROOT
        logger.info(mesg)

    df = psutil.disk_usage(settings.CACHE_ROOT)
    MB = 1024 * 1024

    return {
        'size-in-megabytes': df.used / MB,
        'capacity-in-megabytes': df.total / MB,
        'free': df.free / MB,
        'percent-used': df.percent,
        }


def jobs():

    return dict(
        total=JobSplit.objects.count(),
        running=JobSplit.objects.filter(status=Job.PROCESSING).count(),
        queued=JobSplit.objects.filter(status=Job.QUEUED).count(),
        cancelled=JobSplit.objects.filter(status=Job.CANCELLED).count(),
        skipped=JobSplit.objects.filter(status=Job.SKIPPED).count(),
        completed=JobSplit.objects.filter(status=Job.COMPLETED).count(),
        failed=JobSplit.objects.filter(status=Job.FAILED).count(),
        )


def experiments():

    return dict(
        running=Experiment.objects.filter(status=Experiment.RUNNING).count(),
        scheduled=Experiment.objects.filter(status=Experiment.SCHEDULED).count(),
        ls=Experiment.objects.filter(status__in=(Experiment.RUNNING,
          Experiment.SCHEDULED)).order_by('author__username',
            'toolchain__author__username', 'toolchain__name',
            'toolchain__version', 'name'),
        )
