#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.dispatch import receiver
from django.contrib.auth.models import User
from django.db.models import Q

from actstream import action

from beat.web.team.models import Team

from ..models import Shareable
from ..signals import shared

def send_related_sharing(verb, sender, **kwargs):
    users = kwargs.get('users')
    teams = kwargs.get('teams')

    author = sender.author

    if not users and not teams:
        action.send(author, verb=verb, action_object=sender)

    if users is not None and len(users) > 0:
        # 'users' might be a list of names or a list of DB objects
        if isinstance(users[0], User):
            db_users = users
        else:
            db_users = User.objects.filter(username__in=users)

        for user in db_users:
            action.send(author, verb=verb, action_object=sender, target=user, public=False)


    if teams is not None and len(teams) > 0:
        # 'teams' might be a list of names or a list of DB objects
        db_teams = []

        if isinstance(teams[0], Team):
            db_teams = teams
        else:
            query = None

            for team_full_name in teams:
                try:
                    username, teamname = team_full_name.split('/')
                except ValueError:
                    username = author
                    teamname = team_full_name
                else:
                    if query:
                        query |= Q(owner__username=username, name=teamname)
                    else:
                        query = Q(owner__username=username, name=teamname)

            if query:
                db_teams = Team.objects.filter(query)

        for team in db_teams:
            action.send(author, verb=verb, action_object=sender, target=team, public=False)


@receiver(shared)
def on_shared(sender, **kwargs):
    sharing = sender.sharing
    verb = 'has done nothing'
    public = False

    if hasattr(sender, 'author'):
        if sharing == Shareable.PUBLIC:
            verb = "has made public"
            public = True
        elif sharing == Shareable.SHARED:
            verb = "has shared"
        elif sharing == Shareable.USABLE:
            verb = "open-sourced"
        elif sharing == Shareable.PRIVATE:
            verb = "has privatized"

        if sharing in [Shareable.SHARED, Shareable.USABLE]:
            send_related_sharing(verb, sender, **kwargs)
        else:
            action.send(sender.author, verb=verb, target=sender, public=public)
    else:
        if sharing == Shareable.PUBLIC:
            verb = "was made public"
            public = True
        elif sharing == Shareable.SHARED:
            verb = "was shared"
        elif sharing == Shareable.USABLE:
            verb = "was open-sourced"
        elif sharing == Shareable.PRIVATE:
            verb = "was privatized"

        action.send(sender, verb=verb, public=public)

