/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/* beat.experiments.panels.js

   Implementation of the panels related to the experiments
   */

// Declaration of our namespaces
if (beat === undefined) var beat = {}
if (beat.experiments === undefined) beat.experiments = {}
if (beat.experiments.panels === undefined) beat.experiments.panels = {}


/********************** CLASS: Settings ************************************/

//----------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------
beat.experiments.panels.Settings = function(panel_id, toolchain_name,
    algorithm_mapping, url_prefix, username, name) {

  this.id                 = panel_id;
  this.element            = $('#' + panel_id)[0];
  this.toolchain          = null;
  this.configuration      = null;
  this.datasets           = null;
  this.algorithms         = null;
  this.dataformats        = null;
  this.environments       = null;
  this.environments_all   = null;
  this.smart_datasets     = null;
  this.algorithm_mapping  = algorithm_mapping;
  this.url_prefix         = url_prefix;
  this.current_block      = null;
  this.block_infos        = [];
  this.onAlgorithmChanged = null;
  this.beforeSaveCallback = null;

  if (name) $('#' + panel_id + '_name')[0].value = name;
  $('#' + panel_id + '_globals').hide();

  var panel = this;

  function _scheduleExperiment(queue) {
    // Use the Web API to schedule the experiment
    var data = {
      declaration: panel.configuration.declaration,
    };

    var experiment_name = $('#' + panel.id + '_name')[0].value.trim();
    if (experiment_name !== '')
      data.name = experiment_name;

    var http_method = null;
    var url = url_prefix + '/api/v1/experiments/';

    if (name) {
      http_method = 'PUT';
      url += username + '/' + toolchain_name + '/' + name + '/';
    }
    else {
      data.toolchain = toolchain_name;

      http_method = 'POST';
      url += username + '/';
    }

    if (panel.beforeSaveCallback) {
      if (!panel.beforeSaveCallback(data))
        return;
    }

    $.ajaxSetup({
      beforeSend: function(xhr, settings) {
        var csrftoken = $.cookie('csrftoken');
        xhr.setRequestHeader('X-CSRFToken', csrftoken);
      }
    });

    var d = $.ajax({
      type: http_method,
      url: url,
      data: JSON.stringify(data),
      contentType: "application/json; charset=utf-8",
      dataType: "json",
    });

    d.done(function(data) {
      if (queue) {
        window.location = url_prefix + '/experiments/' + data.name.split('/')[0] + '/';
        return;
      }

      var d2 = $.ajax({
        type: 'POST',
        url: data.url + 'start/',
        data: JSON.stringify(data),
        contentType: "application/json; charset=utf-8",
        dataType: "json",
      });

      d2.done(function(data) {
        window.location = url_prefix + '/experiments/' + data.name + '/';
      });

      d2.fail(function(data, status) {
        $('#' + panel.id + ' a.queue_experiment').removeClass('disabled');
        $('#' + panel.id + ' a.run_experiment').removeClass('disabled');
        $('#' + panel.id + ' a.run_experiment i.fa').removeClass('fa-spin');
        $('#' + panel.id + ' a.run_experiment span').text('Go!');
        process_error(data, status, url_prefix + '/experiments/' + username + '/');
      });
    });

    d.fail(function(data, status) {
      $('#' + panel.id + ' a.queue_experiment').removeClass('disabled');
      $('#' + panel.id + ' a.run_experiment').removeClass('disabled');
      $('#' + panel.id + ' a.run_experiment i.fa').removeClass('fa-spin');
      $('#' + panel.id + ' a.run_experiment span').text('Go!');
      process_error(data, status);
    });
  }


  $('#' + this.id + ' a.run_experiment').click(function() {
    if ($(this).hasClass('disabled'))
      return false;

    _scheduleExperiment(false);

    $('#' + panel.id + ' a.queue_experiment').addClass('disabled');
    $('#' + panel.id + ' a.run_experiment').addClass('disabled');
    $('#' + panel.id + ' a.run_experiment i.fa').addClass('fa-spin');
    $('#' + panel.id + ' a.run_experiment span').text('Submitting...');

    return false;
  });


  $('#' + this.id + ' a.queue_experiment').click(function() {
    if ($(this).hasClass('disabled'))
      return false;

    _scheduleExperiment(true);

    $('#' + panel.id + ' a.queue_experiment').addClass('disabled');
    $('#' + panel.id + ' a.run_experiment').addClass('disabled');

    return false;
  });


  $('#' + this.id + '_name').keyup(function(event) {
    panel._updateRunButton();
  });
}


//----------------------------------------------------------------------
// Populate the panel using the description of a toolchain
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype.initialize = function(toolchain, configuration,
    dataformats, datasets,
    algorithms, environments)
{
  this.toolchain        = toolchain;
  this.configuration    = configuration;
  this.dataformats      = dataformats;
  this.datasets         = datasets;
  this.environments_all = environments;
  this.algorithms       = algorithms;

  // Separate the environments by languages
  this.environments = {};

  var languages = environments.languages();
  for (var i = 0; i < languages.length; ++i)
  {
    var language = languages[i];
    this.environments[language] = environments.filter(language);
  }

  // Update the environments
  var components = [].concat(this.toolchain.blocks, this.toolchain.analyzers);
  for (var i = 0; i < components.length; ++i)
  {
    var component = components[i];

    var configuration = this.configuration.component(component.name);

    if ((configuration != null) && (configuration.environment != null))
    {
      if (!this.environments_all.contains(configuration.environment.name, configuration.environment.version))
        this.configuration.setBlockEnvironment(component.name, this.environments_all.get(configuration.environment.name));
    }
  }

  this.algorithm_mapping.dataformats = dataformats;

  $('#' + this.id + ' div.section').show();
  $('#' + this.id + ' div.buttons').show();

  this._initializeBlocks();
  this._initializeAnalyzers();
  this._initializeDatasets();

  this._updateRunButton();
}


//-----------------------------


beat.experiments.panels.Settings.prototype._initializeBlocks = function()
{
  this._initializeAlgorithmComponents(this.toolchain.blocks,
      $('#' + this.id + '_blocks table tbody')[0]);
}


//-----------------------------


beat.experiments.panels.Settings.prototype._initializeAnalyzers = function()
{
  this._initializeAlgorithmComponents(this.toolchain.analyzers,
      $('#' + this.id + '_analyzers table tbody')[0]);
}


//-----------------------------


beat.experiments.panels.Settings.prototype._initializeAlgorithmComponents = function(components, table)
{
  // Sort the components
  var origin = {
    col: null,
    row:  null,
  };

  for (var i = 0; i < components.length; ++i) {
    var component = components[i];
    component['representation'] = this.toolchain.declaration.representation.blocks[component.name];

    if ((origin.col === null) || (origin.col > component.representation.col))
      origin.col = component.representation.col;

    if ((origin.row === null) || (origin.row > component.representation.row))
      origin.row = component.representation.row;
  }

  function _custom_cmp(a, b)
  {
    var dist_a = Math.pow(a.representation.col - origin.col, 2) + 0.1*Math.abs(a.representation.row - origin.row);
    var dist_b = Math.pow(b.representation.col - origin.col, 2) + 0.1*Math.abs(b.representation.row - origin.row);

    return dist_a - dist_b;
  }

  var components_sorted_by_position = [].concat(components);
  components_sorted_by_position.sort(_custom_cmp);

  var sorted_components = [];

  while (components_sorted_by_position.length > 0)
  {
    var reference_component = components_sorted_by_position[0];
    components_sorted_by_position.splice(0, 1);

    sorted_components.push(reference_component);

    var i = 0;
    while (i < components_sorted_by_position.length)
    {
      var component = components_sorted_by_position[i];

      if (reference_component.name.score(component.name, 0.5) > 0.2)
      {
        components_sorted_by_position.splice(i, 1);
        sorted_components.push(component);
      }
      else
      {
        ++i;
      }
    }
  }


  var panel = this;

  for (var i = 0; i < sorted_components.length; ++i)
  {
    var block_entry = {
      block:             null,
      algorithm:         null,
      smart_mapping:     null,
      selection_row:     null,
      algorithm_element: null,
      delete_button:     null,
      parameters_button: null,
      reset_button:      null,
      mapping_button:    null,
      view_button:       null,
      description_row:   null,
      description_cell:  null,
      parameters_panel:  null,
    };

    block_entry.block = sorted_components[i];
    var block_configuration = this.configuration.component(block_entry.block.name);

    block_entry.selection_row = document.createElement('tr');
    table.appendChild(block_entry.selection_row);

    $(block_entry.selection_row).addClass('active');

    var td = document.createElement('th');
    $(td).text(block_entry.block.name);
    block_entry.selection_row.appendChild(td);

    td = document.createElement('td');
    block_entry.selection_row.appendChild(td);

    block_entry.algorithm_element = $(document.createElement('select'));
    block_entry.algorithm_element.addClass('form-control input-sm');
    var opt = $(document.createElement('option'));
    opt.val('');
    opt.attr('disabled', true);
    opt.attr('selected', true);
    opt.text('Select an algorithm...');
    block_entry.algorithm_element.append(opt);
    block_entry.algorithm_element[0].entry = block_entry;
    //we use the DOM object to keep compatibility with the rest of the code
    td.appendChild(block_entry.algorithm_element[0]);
    block_entry.algorithm_element.chosen({
      allow_single_deselect: true,
      search_contains: true,
    });

    block_entry.algorithm_element = block_entry.algorithm_element[0];

    //builds the button group using the button group template
    td = document.createElement('td');
    td.className = 'action-buttons';
    block_entry.selection_row.appendChild(td);

    var button_group = $('.algorithm-buttons').clone();
    td.appendChild(button_group[0]);

    block_entry.delete_button = button_group.find('button:eq(0)')[0];
    block_entry.delete_button.entry = block_entry;

    block_entry.mapping_button = button_group.find('button:eq(1)')[0];
    block_entry.mapping_button.entry = block_entry;

    block_entry.reset_button = button_group.find('button:eq(2)')[0];
    block_entry.reset_button.entry = block_entry;

    block_entry.parameters_button = button_group.find('button:eq(3)')[0];
    block_entry.parameters_button.entry = block_entry;

    block_entry.view_button = button_group.find('button:eq(4)')[0];
    block_entry.view_button.entry = block_entry;

    if (block_configuration && block_configuration.algorithm)
    {
      $(block_entry.algorithm_element).find('option').remove();
      var option = $(document.createElement('option'));
      option.val(block_configuration.algorithm);
      option.text(block_configuration.algorithm);
      option.attr('selected', true);
      $(block_entry.algorithm_element).append(option);
      $(block_entry.algorithm_element).trigger("chosen:updated");

      block_entry.algorithm = this.algorithms.get(block_configuration.algorithm);

      for (var j = 0; j < block_entry.block.nbInputs(); ++j)
      {
        var block_input          = block_entry.block.inputs[j];
        var algorithm_input_name = this.configuration.algorithmInputName(block_entry.block.name, block_input.name);
        var algorithm_input      = block_entry.algorithm.input(algorithm_input_name);
        block_input.dataformat   = algorithm_input.dataformat;
      }

      for (var j = 0; j < block_entry.block.nbOutputs(); ++j)
      {
        var block_output          = block_entry.block.outputs[j];
        var algorithm_output_name = this.configuration.algorithmOutputName(block_entry.block.name, block_output.name);
        var algorithm_output      = block_entry.algorithm.output(algorithm_output_name);
        block_output.dataformat   = algorithm_output.dataformat;
      }

      this._updateDescriptionControls(block_entry);
      this._updateParametersControls(block_entry);
    }
    else
    {
      $(block_entry.delete_button).hide();
      $(block_entry.parameters_button).hide();
      $(block_entry.reset_button).hide();
      $(block_entry.mapping_button).hide();
      $(block_entry.view_button).hide();
    }


    $(block_entry.algorithm_element).on('chosen:showing_dropdown', function() {
      panel.current_block = this.entry.block.name;

      // Determine the block "signature" (data formats usage on inputs and outputs)
      var block_inputs_signature  = beat.experiments.utils.getBlockInputSignature(this.entry.block);
      var block_outputs_signature = beat.experiments.utils.getBlockOutputSignature(this.entry.block);

      // Iterate through all the algorithms to find the compatible ones
      var iterator = panel.algorithms.iterator();

      $(this).find('option').remove();

      if (!this.entry.algorithm) { //no previous selection
        var opt = $(document.createElement('option'));
        opt.val('');
        opt.attr('disabled', true);
        opt.attr('selected', true);
        opt.text('Select an algorithm...');
        $(this).append(opt);
      }

      while (iterator.hasMoreData()) {
        var algorithm = iterator.next();
        if (algorithm.duplicate_of !== null)
          continue;

        // If compatible, add the algorithm to the control allowing to select the algorithm
        var algorithm_inputs_signature = beat.experiments.utils.getEntryPointsSignature(algorithm.inputs);
        var algorithm_outputs_signature = beat.experiments.utils.getEntryPointsSignature(algorithm.outputs);

        var smart_mapping = beat.experiments.utils.analyzeCompatibility(
            block_inputs_signature, block_outputs_signature,
            algorithm_inputs_signature, algorithm_outputs_signature,
            panel.dataformats);
        if (smart_mapping) {
          var opt = $(document.createElement('option'));
          opt.val(algorithm.name);
          opt[0].smart_mapping = smart_mapping;
          if (this.entry.algorithm && this.entry.algorithm.name == algorithm.name) {
            opt.attr('selected', true);
          }

          var div = $(document.createElement('div'));
          opt.append(div);
          div.text(algorithm.name);
          if (algorithm.short_description) {
            var help = $(document.createElement('span')).addClass('help');
            help.text(' (' + algorithm.short_description + ')');
            div.append(help);
          }

          $(this).append(opt);
        }
      }

      $(this).trigger("chosen:updated");
    });

    $(block_entry.algorithm_element).on('chosen:hiding_dropdown', function() {
      var val = $(this).val();
      var opt = $(this).find('option:selected');
      if (val) {
        panel._setBlock(this.entry, val, opt[0].smart_mapping);
        opt.text(val);
        $(this).trigger('chosen:updated');
      }
    });

    $(block_entry.delete_button).click(function() {
      panel._resetBlock(this.entry);
    });


    $(block_entry.parameters_button).click(function() {
      panel._displayParametersControls(this.entry);
    });


    $(block_entry.reset_button).click(function() {
      if (this.entry.parameters_panel)
      {
        this.entry.parameters_panel.destroy();
        this.entry.parameters_panel = null;

        $(this.entry.parameters_button).show();
        $(this).hide();

        panel.configuration.resetComponentParameters(this.entry.block.name);
        panel.configuration.resetBlockEnvironmentAndQueue(this.entry.block.name, this.entry.block.isAnalyzer());
      }
    });


    $(block_entry.mapping_button).click(function() {
      if (!this.entry.smart_mapping)
      {
        // Determine the block "signature" (data formats usage on inputs and
        // outputs)
        var block_inputs_signature  = beat.experiments.utils.getBlockInputSignature(this.entry.block);
        var block_outputs_signature = beat.experiments.utils.getBlockOutputSignature(this.entry.block);

        // Determine the algorithm "signature"
        var algorithm_inputs_signature = beat.experiments.utils.getEntryPointsSignature(this.entry.algorithm.inputs);
        var algorithm_outputs_signature = beat.experiments.utils.getEntryPointsSignature(this.entry.algorithm.outputs);

        this.entry.smart_mapping = beat.experiments.utils.analyzeCompatibility(
            block_inputs_signature, block_outputs_signature,
            algorithm_inputs_signature, algorithm_outputs_signature,
            panel.dataformats);
      }

      panel.algorithm_mapping.display(this.entry.block, this.entry.algorithm,
          panel.configuration, this.entry.smart_mapping);
    });


    $(block_entry.view_button).click(function() {
      window.open(panel.url_prefix + '/algorithms/' + this.entry.algorithm.name + '/', '_blank');
    });

    this.block_infos.push(block_entry);
  }
}


//-----------------------------


beat.experiments.panels.Settings.prototype._initializeDatasets = function()
{
  var panel = this;

  var table = $('#' + this.id + '_datasets table tbody')[0];

  this.smart_datasets = new beat.experiments.utils.SmartDatasetList();
  var configured = false;

  //setup buttons (independently of current configuration)
  $('#' + this.id + '_datasets a#dataset-advanced').click(function() {
    $(this).hide();
    $('#' + panel.id + '_datasets table tr:not(.simple)').show();
    $('#' + panel.id + '_datasets table tr.simple').hide();
    $('#' + panel.id + '_datasets a#dataset-reset').show();
    return false;
  });

  $('#' + this.id + '_datasets a#dataset-reset').click(function() {
    return panel._resetDatasetPanel(this);
  });

  // Add the controls for each dataset block
  for (var i = 0; i < this.toolchain.nbDatasets(); ++i)
  {
    var dataset_block = this.toolchain.datasets[i];

    if (!configured)
    {
      var dataset_configuration = this.configuration.dataset(dataset_block.name);

      if (dataset_configuration)
        configured = true;
    }

    var tr = document.createElement('tr');
    table.appendChild(tr);

    var td = document.createElement('th');
    $(td).text(dataset_block.name);
    tr.appendChild(td);

    td = document.createElement('td');
    tr.appendChild(td);

    var select = document.createElement('select');
    select.className = 'form-control input-sm unselected';
    select.id = dataset_block.name;
    select.available_options = [];
    td.appendChild(select);

    td = document.createElement('td');
    td.className = 'action-buttons';
    tr.appendChild(td);

    var delete_button = $('.algorithm-buttons>button:eq(0)').clone();
    delete_button.attr('title', 'Reset the dataset');
    delete_button.attr('id', dataset_block.name + '__delete_button');
    delete_button.removeClass('btn-xs');
    delete_button.addClass('btn-sm');
    td.appendChild(delete_button[0]);
    delete_button.hide();

    var first_option = $(document.createElement('option'));
    first_option.text('Select one dataset...');
    first_option.val('');
    first_option.attr('selected', true);
    first_option.disable();
    $(select).append(first_option);
    $(select).chosen({
      allow_single_deselect: true,
      search_contains: true,
    });

    // Search for all the datasets compatibles with the block
    var iterator = this.datasets.iterator();
    while (iterator.hasMoreData())
    {
      var dataset = iterator.next();

      var result = dataset.isCompatible(dataset_block, this.dataformats);
      if (!result.compatible)
        continue;

      var option = document.createElement('option');
      $(option).val(dataset.name);
      $(option).text(dataset.name);

      if (result.perfect) $(option).addClass('perfect');
      if (result.usable) $(select).append($(option));

      select.available_options.push(option);

      // Memorize the information for the simple mode
      this.smart_datasets.add(dataset.database, dataset.protocol,
          dataset_block.name, dataset.set, result);
    }
    $(select).trigger('chosen:updated');

    // Event handling
    $(select).on('change', function() {
      var val = $(this).val();
      if (val && val !== '') {
        var dataset_configuration = panel.configuration.dataset(this.id);
        if (!dataset_configuration) {
          $(this).removeClass('unselected');
          $('#' + panel.id + ' #' + this.id + '__delete_button').show();
        }

        var parts = val.split('/');
        panel.configuration.setDatasetDetails(this.id, parts[0] + '/' + parts[1], parts[2], parts[3]);

        var dataset_block = panel.toolchain.dataset(this.id);

        var dataset = panel.datasets.get(val);

        for (var j = 0; j < dataset_block.nbOutputs(); ++j)
        {
          var dataset_output = dataset.output(dataset_block.outputs[j].name);
          dataset_block.outputs[j].dataformat = dataset_output.dataformat;
        }
      }

      panel._updateRunButton();
    });


    delete_button.click(function() {
      panel._resetDataset(this.id.replace('__delete_button', ''));
    });
  }


  // Datasets not configured: setup the simple mode controls
  if (!configured)
  {
    $('#' + this.id + '_datasets a#dataset-reset').hide();

    // Attempt to find the best configuration for each dataset block for each
    // database/protocol
    if (this.smart_datasets.finish(this.toolchain.nbDatasets()))
    {
      // Setup the controls
      var db_selector = $('#' + this.id + '_datasets select#database_simple')[0];

      db_selector.available_options = [];

      var database_names = this.smart_datasets.databaseNames();
      for (var i = 0; i < database_names.length; ++i)
      {
        var option   = document.createElement('option');
        option.value = database_names[i];
        option.text  = option.value;

        db_selector.available_options.push(option);
        db_selector.appendChild(option);
      }

      // Event handling
      $(db_selector).change(function() {
        var protocol_selector = $('#' + panel.id + '_datasets select#protocol_simple')[0];

        if ((this.value != '') && (this.options[0].value == ''))
        {
          $(this).removeClass('unselected');
          $('#' + panel.id + '_datasets #database_simple_delete_button').show();
          $(protocol_selector).enable();
        }

        $(protocol_selector).find('option').remove();
        protocol_selector.available_options = [];

        $('#' + panel.id + '_datasets #protocol_simple_delete_button').click();

        var protocol_names = panel.smart_datasets.protocolNames(this.value);
        for (var i = 0; i < protocol_names.length; ++i)
        {
          var option   = document.createElement('option');
          option.value = protocol_names[i];
          option.text  = option.value;

          protocol_selector.available_options.push(option);

          if (panel.smart_datasets.isProtocolUsable(this.value, option.value))
            protocol_selector.appendChild(option);
        }
      });

      $('#' + panel.id + '_datasets select#protocol_simple').change(function() {
        if ((this.value != '') && (this.options[0].value == ''))
        {
          $(this).removeClass('unselected');
          $('#' + panel.id + '_datasets #protocol_simple_delete_button').show();
        }

        var database_name = $('#' + panel.id + '_datasets select#database_simple')[0].value;
        var protocol = panel.smart_datasets.protocol(database_name, this.value);
        var block_names = Object.keys(protocol.blocks);
        for (var i = 0; i < block_names.length; ++i)
        {
          var block_name = block_names[i];
          var selector = $('#' + panel.id + '_datasets select#' + block_name)[0];
          $(selector).val(database_name + '/' + this.value + '/' + protocol.blocks[block_name].set).change();
          $(selector).trigger('chosen:updated');
        }

        panel._updateRunButton();
      });

      $('#' + panel.id + '_datasets #database_simple_delete_button').click(function() {
        var protocol_selector = $('#' + panel.id + '_datasets select#protocol_simple')[0];

        $(protocol_selector).find('option').remove();
        $(protocol_selector).disable();

        protocol_selector.available_options = [];
        $(protocol_selector).addClass('unselected');

        var option       = document.createElement('option');
        option.value     = '';
        option.text      = 'Select one protocol...';
        $(option).addClass('unselected');
        $(option).attr('selected', true);
        $(option).disable();
        protocol_selector.appendChild(option);

        $('#' + panel.id + '_datasets #protocol_simple_delete_button').click();

        var database_selector = $('#' + panel.id + '_datasets select#database_simple')[0];

        if (database_selector.options[0].value != '')
        {
          option           = document.createElement('option');
          option.value     = '';
          option.text      = 'Select one database...';
          $(option).attr('selected', true);
          $(option).addClass('unselected');
          $(option).disable();
          $(option).insertBefore(database_selector.options[0]);
        }

        $(database_selector).addClass('unselected');
        database_selector.selectedIndex = 0;

        $(this).hide();
      });

      $('#' + panel.id + '_datasets #protocol_simple_delete_button').click(function() {
        var protocol_selector = $('#' + panel.id + '_datasets select#protocol_simple')[0];

        if (protocol_selector.options.length == 0) {
          var option       = document.createElement('option');
          option.value     = '';
          option.text      = 'Select one protocol...';
          $(option).attr('selected', true);
          $(option).disable();
          $(option).addClass('unselected');
          protocol_selector.appendChild(option);
        }

        $(protocol_selector).addClass('unselected');
        protocol_selector.selectedIndex = 0;

        $(this).hide();

        for (var i = 0; i < panel.toolchain.nbDatasets(); ++i)
        {
          var dataset_block = panel.toolchain.datasets[i];
          panel._resetDataset(dataset_block.name);
        }

        panel._updateRunButton();
      });

      // Update the interface
      $('#' + this.id + '_datasets table tr:not(.simple)').hide();
      $('#' + this.id + '_datasets table tr.simple span.icon').hide();
      $('#' + this.id + '_datasets select#protocol_simple').disable();
    }
    else
    {
      this.smart_datasets = null;
    }
  }

  // Datasets already configured OR simple mode not possible: hide simple mode controls
  if (configured || (this.smart_datasets === null))
  {
    this.smart_datasets = null;
    $('#' + this.id + '_datasets a#dataset-advanced').hide();
    $('#' + this.id + '_datasets table tr.simple').hide();

    // Build a table that can be used in case of database update
    var databases_update_table = {};
    for (var i = 0; i < this.datasets.length; ++i)
    {
      var name = this.datasets.getByIndex(i).database;
      var parts = name.split('/');

      if (databases_update_table[parts[0]] === undefined)
        databases_update_table[parts[0]] = name;
    }

    for (var i = 0; i < this.toolchain.nbDatasets(); ++i)
    {
      var dataset_block = this.toolchain.datasets[i];
      var dataset_configuration = this.configuration.dataset(dataset_block.name);

      if (!dataset_configuration)
        continue;

      var configured_dataset = databases_update_table[dataset_configuration.database.split('/')[0]] + '/' +
        dataset_configuration.protocol + '/' +
        dataset_configuration.set;

      var dataset = this.datasets.get(configured_dataset);

      if (!configured_dataset.startsWith(dataset_configuration.database))
      {
        var parts = configured_dataset.split('/');
        this.configuration.setDatasetDetails(dataset_block.name, parts[0] + '/' + parts[1], parts[2], parts[3]);
      }

      var selector = $('#' + panel.id + '_datasets select#' + dataset_block.name)[0];

      for (var j = 0; j < selector.options.length; ++j)
      {
        if (selector.options[j].value == configured_dataset)
        {
          selector.selectedIndex = j;
          $(selector).removeClass('unselected');
          $(selector).trigger("chosen:updated");

          $('#' + panel.id + '_datasets #' + dataset_block.name + '__delete_button').show();

          for (var k = 0; k < dataset_block.nbOutputs(); ++k)
          {
            var dataset_output = dataset.output(dataset_block.outputs[k].name);
            dataset_block.outputs[k].dataformat = dataset_output.dataformat;
          }

          break;
        }
      }
    }
  }
}


//----------------------------------------------------------------------
// Retrieves the internal details about the specified block
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._blockEntry = function(block_name)
{
  return this.block_infos.filter(function(entry) { return entry.block.name == block_name; })[0];
}


//----------------------------------------------------------------------
// Called when the user selected an algorithm for a block
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._onAlgorithmSelected = function(by_user)
{
  var block_entry = this._blockEntry(this.current_block);

  // Update the algorithm in the entry
  var algorithm_name    = this.configuration.componentAlgorithm(this.current_block);
  block_entry.algorithm = this.algorithms.get(algorithm_name);

  // Update the environment if necessary
  var block_env = this.configuration.blockEnvironment(this.current_block);
  var default_env = this.configuration.defaultEnvironment();

  if (!this.environments[block_entry.algorithm.language].contains(block_env.name, block_env.version))
  {
      if (!this.environments[block_entry.algorithm.language].contains(default_env.name, default_env.version))
      {
          var env = this.environments[block_entry.algorithm.language].getByIndex(0);

          var config_env = {
            name: env.name,
            version: env.version,
          };
          this.configuration.setBlockEnvironment(this.current_block, config_env);
      }
      else
      {
          this.configuration.resetBlockEnvironmentAndQueue(this.current_block);
      }
  }
  else if ((block_env.name == default_env.name) && (block_env.version == default_env.version))
  {
      this.configuration.resetBlockEnvironmentAndQueue(this.current_block);
  }

  // Update the selection controls
  $(block_entry.algorithm_element).find('option').remove();
  var option = $(document.createElement('option'));
  option.val(algorithm_name);
  option.text(algorithm_name);
  option.attr('selected', true);
  $(block_entry.algorithm_element).append(option);
  $(block_entry.algorithm_element).trigger("chosen:updated");

  if (!by_user)
    $(block_entry.algorithm_element).addClass('automatic');
  else
    $(block_entry.algorithm_element).removeClass('automatic');

  $(block_entry.delete_button).show();
  $(block_entry.parameters_button).show();
  $(block_entry.reset_button).hide();
  $(block_entry.mapping_button).show();
  $(block_entry.view_button).show();

  // Update the description, parameters and datasets controls
  this._updateDescriptionControls(block_entry);
  this._updateParametersControls(block_entry);
  this._updateDatasetsControls(block_entry);

  this.current_block = null;
  this._updateRunButton();
}


//---------------------------------------


beat.experiments.panels.Settings.prototype._updateDatasetsControls = function(block_entry)
{
  // Retrieve the list of dataset blocks connected to the block
  var datasets_to_process = [];
  for (var i = 0; i < block_entry.block.nbInputs(); ++i)
  {
    var connected_block = block_entry.block.inputs[i].connections[0].output.block;
    if (connected_block.isDataset())
    {
      if (datasets_to_process.indexOf(connected_block) == -1)
        datasets_to_process.push(connected_block);
    }
  }

  // Update the dataset selectors to keep only the compatible ones
  for (var i = 0; i < datasets_to_process.length; ++i)
  {
    var dataset_block = datasets_to_process[i];

    var selector = $('#' + this.id + ' select#' + dataset_block.name)[0];
    var value = selector.value;

    if (value == '')
      $(selector).find('option:not(.unselected)').remove();
    else
      $(selector).find('option').remove();

    var index_to_select = 0;
    for (var j = 0; j < selector.available_options.length; ++j)
    {
      var option = selector.available_options[j];

      var dataset = this.datasets.get(option.value);

      var result = dataset.isCompatible(dataset_block, this.dataformats);

      if (result.usable)
      {
        selector.appendChild(option);

        if (option.value == value)
          index_to_select = selector.options.length - 1;
      }

      if (this.smart_datasets !== null)
      {
        this.smart_datasets.update(dataset.database, dataset.protocol, dataset_block.name,
            dataset.set, result.usable);
      }
    }

    selector.selectedIndex = index_to_select;
  }

  // Update the simple mode selectors to keep only the compatible databases/protocols
  if (this.smart_datasets !== null)
  {
    var db_selector = $('#' + this.id + ' select#database_simple')[0];
    var protocol_selector = $('#' + this.id + ' select#protocol_simple')[0];
    var counter = 0;

    var db_value = db_selector.value;
    var protocol_value = protocol_selector.value;

    if (db_value == '')
      $(db_selector).find('option:not(.unselected)').remove();
    else
      $(db_selector).find('option').remove();

    if (protocol_value == '')
      $(protocol_selector).find('option:not(.unselected)').remove();
    else
      $(protocol_selector).find('option').remove();

    var index_to_select = 0;
    for (var j = 0; j < db_selector.available_options.length; ++j)
    {
      var option = db_selector.available_options[j];

      if (this.smart_datasets.isDatabaseUsable(option.value))
      {
        db_selector.appendChild(option);
        ++counter;

        if (option.value == db_value)
          index_to_select = db_selector.options.length - 1;
      }
    }

    db_selector.selectedIndex = index_to_select;

    if (counter == 0)
    {
      $('#' + this.id + '_datasets a#dataset-advanced').click();
    }
    else if (protocol_selector.available_options && (protocol_selector.available_options.length > 0))
    {
      counter = 0;
      index_to_select = 0;

      for (var j = 0; j < protocol_selector.available_options.length; ++j)
      {
        var option = protocol_selector.available_options[j];

        if (this.smart_datasets.isProtocolUsable(db_value, option.value))
        {
          protocol_selector.appendChild(option);
          ++counter;

          if (option.value == protocol_value)
            index_to_select = protocol_selector.options.length - 1;
        }
      }

      protocol_selector.selectedIndex = index_to_select;

      if (counter == 0)
        $('#' + this.id + '_datasets a#dataset-advanced').click();
    }
  }
}


//---------------------------------------


beat.experiments.panels.Settings.prototype._displayParametersControls = function(block_entry)
{
  block_entry.parameters_panel = new beat.experiments.panels.BlockParameters(
      (block_entry.description_row ? block_entry.description_row : block_entry.selection_row),
      this.configuration,
      block_entry.block.name,
      block_entry.algorithm,
      this.environments[block_entry.algorithm.language]
      );

  $(block_entry.reset_button).show();
  $(block_entry.parameters_button).hide();
}


//---------------------------------------


beat.experiments.panels.Settings.prototype._hideParametersControls = function(block_entry)
{
  $(block_entry.parameters_button).show();
  $(block_entry.reset_button).hide();

  if (block_entry.parameters_panel)
  {
    block_entry.parameters_panel.destroy();
    block_entry.parameters_panel = null;
  }
}


//---------------------------------------


beat.experiments.panels.Settings.prototype._updateParametersControls = function(block_entry)
{
  this._hideParametersControls(block_entry);
  var block_environment = this.configuration.blockEnvironment(block_entry.block.name, block_entry.block.isAnalyzer());
  var default_environment = this.configuration.defaultEnvironment();

  if (((block_entry.algorithm.parameters != null) && (block_entry.algorithm.parameters.length > 0) &&
        this.configuration.isParametrized(block_entry.block.name)) ||
      (block_environment.name != default_environment.name || block_environment.version != default_environment.version) ||
      (this.configuration.blockQueue(block_entry.block.name, block_entry.block.isAnalyzer()) != this.configuration.defaultQueue()) ||
      (this.configuration.blockNbSlots(block_entry.block.name, block_entry.block.isAnalyzer()) > 1))
  {
    this._displayParametersControls(block_entry);
  }
  else
  {
    this.configuration.resetComponentParameters(block_entry.block.name);
  }
}


//---------------------------------------


beat.experiments.panels.Settings.prototype._updateDescriptionControls = function(block_entry)
{
  if (block_entry.algorithm && block_entry.algorithm.short_description)
  {
    if (block_entry.description_row == null)
    {
      block_entry.description_row = document.createElement('tr');
      block_entry.description_row.className = 'help';
      block_entry.description_cell = document.createElement('td');
      $(block_entry.description_cell).attr('colspan', 3);
      block_entry.description_row.appendChild(block_entry.description_cell);

      $(block_entry.description_row).insertAfter(block_entry.selection_row);
    }

    $(block_entry.description_cell).text(block_entry.algorithm.short_description);
  }
  else
  {
    if (block_entry.description_row != null)
    {
      block_entry.description_row.parentElement.removeChild(block_entry.description_row);

      block_entry.description_row = null;
      block_entry.description_cell = null;
    }
  }
}

//----------------------------------------------------------------------
// Resets discrete datasets so the user can use the smart selector again
//
// Note: Cannot handle advance -> simple mode switch!!!
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._resetDatasetPanel = function(btn) {
  $(btn).hide();
  $('#' + this.id + '_datasets table tr:not(.simple)').hide();
  $('#' + this.id + '_datasets table tr.simple').show();
  $('#' + this.id + '_datasets a#dataset-advanced').show();

  for (var i = 0; i < this.toolchain.datasets.length; ++i) {
    var block_name = this.toolchain.datasets[i].name;
    $('#' + this.id + '_datasets #' + block_name + '__delete_button').click();
  }

  $('#' + this.id + '_datasets #database_simple_delete_button').click();
  return false;
}

//----------------------------------------------------------------------
// Update the display of the 'run' button
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._updateRunButton = function()
{
  if (($('#' + this.id + '_name')[0].value.trim().length > 0) &&
      (this.configuration.nbBlocksConfigured() == this.toolchain.declaration.blocks.length) &&
      (this.configuration.nbDatasetsConfigured() == this.toolchain.declaration.datasets.length) &&
      (this.configuration.nbAnalyzersConfigured() == this.toolchain.declaration.analyzers.length))
  {
    $('#' + this.id + ' a.queue_experiment').removeClass('disabled');
    $('#' + this.id + ' a.run_experiment').removeClass('disabled');
  }
  else
  {
    $('#' + this.id + ' a.queue_experiment').addClass('disabled');
    $('#' + this.id + ' a.run_experiment').addClass('disabled');
  }
}

//----------------------------------------------------------------------
// Sets a block (algorithm chosen)
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._setBlock = function(block_entry, algorithm_name, smart_mapping) {

  var panel = this;

  function _updateMapping(block_entry, automatically_configured_blocks) {
    for (var i = 0; i < block_entry.block.nbInputs(); ++i) {
      var input = block_entry.block.inputs[i];

      var algorithm_input = panel.configuration.algorithmInputName(block_entry.block.name, input.name)

        for (var j = 0; j < automatically_configured_blocks.length; ++j) {
          var block = automatically_configured_blocks[j];
          panel.configuration.setComponentInputMapping(block.name, algorithm_input, input.name)
            block.input(input.name).dataformat = input.dataformat;
        }
    }

    for (var i = 0; i < block_entry.block.nbOutputs(); ++i) {
      var output = block_entry.block.outputs[i];

      var algorithm_output = panel.configuration.algorithmOutputName(block_entry.block.name, output.name)

        for (var j = 0; j < automatically_configured_blocks.length; ++j) {
          var block = automatically_configured_blocks[j];
          panel.configuration.setComponentOutputMapping(block.name, algorithm_output, output.name)
            block.output(output.name).dataformat = output.dataformat;
        }
    }
  }

  var previous_algorithm_name = panel.configuration.componentAlgorithm(panel.current_block);

  var block_entry = panel._blockEntry(panel.current_block);
  block_entry.smart_mapping = smart_mapping;

  if (algorithm_name == previous_algorithm_name) {
    $(block_entry.algorithm_element).removeClass('automatic');
    this.current_block = null;
    return;
  }

  panel.configuration.setComponentAlgorithm(block_entry.block, algorithm_name);

  var algorithm = panel.algorithms.get(algorithm_name);

  var ambiguous = panel.configuration.completeComponentMapping(block_entry.block, algorithm,
      panel.dataformats,
      block_entry.smart_mapping
      );

  panel._onAlgorithmSelected(true);

  if (panel.onAlgorithmChanged)
    panel.onAlgorithmChanged(algorithm_name, previous_algorithm_name);

  // Auto-assign the same algorithm to other similar-looking not-configured
  // blocks
  var automatically_configured_blocks = [];
  if (previous_algorithm_name === null) {
    for (var i = 0; i < panel.toolchain.nbBlocks(); ++i) {
      var block = panel.toolchain.blocks[i];

      var configured_algorithm = panel.configuration.componentAlgorithm(block.name);
      if (configured_algorithm !== null)
        continue;

      if ((block.nbInputs() != block_entry.block.nbInputs()) ||
          (block.nbOutputs() != block_entry.block.nbOutputs()) ||
          (block.inputNames().toString() != block_entry.block.inputNames().toString()) ||
          (block.outputNames().toString() != block_entry.block.outputNames().toString()))
        continue;

      var block_inputs_signature  = beat.experiments.utils.getBlockInputSignature(block);
      var block_outputs_signature = beat.experiments.utils.getBlockOutputSignature(block);

      var algorithm_inputs_signature  = beat.experiments.utils.getEntryPointsSignature(algorithm.inputs);
      var algorithm_outputs_signature = beat.experiments.utils.getEntryPointsSignature(algorithm.outputs);

      if (beat.experiments.utils.analyzeCompatibility(block_inputs_signature, block_outputs_signature,
            algorithm_inputs_signature, algorithm_outputs_signature,
            panel.dataformats)) {
        panel.current_block = block.name;
        panel.configuration.setComponentAlgorithm(block, algorithm_name);
        panel._onAlgorithmSelected(false);

        automatically_configured_blocks.push(block);
      }
    }
  }

  // Mapping
  if (ambiguous) {
    panel.algorithm_mapping.onClose = function() {
      _updateMapping(block_entry, automatically_configured_blocks);
      panel.algorithm_mapping.onClose = null;
    };

    panel.algorithm_mapping.display(block_entry.block, algorithm, panel.configuration,
        block_entry.smart_mapping);
  }
  else {
    _updateMapping(block_entry, automatically_configured_blocks);
  }
}


//----------------------------------------------------------------------
// Reset a block (no algorithm)
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._resetBlock = function(block_entry)
{
  // Update the selection controls
  $(block_entry.algorithm_element).find('option').remove();
  var opt = $(document.createElement('option'));
  opt.val('');
  opt.attr('disabled', true);
  opt.attr('selected', true);
  opt.text('Select an algorithm...');
  $(block_entry.algorithm_element).append(opt);
  $(block_entry.algorithm_element).trigger("chosen:updated");

  $(block_entry.algorithm_element).removeClass('automatic');
  $(block_entry.delete_button).hide();
  $(block_entry.parameters_button).hide();
  $(block_entry.reset_button).hide();
  $(block_entry.mapping_button).hide();
  $(block_entry.view_button).hide();

  // Update the algorithm in the entry & configuration
  var previous_algorithm = block_entry.algorithm.name;

  if (block_entry.block.isProcessingBlock())
    this.configuration.resetBlock(block_entry.block.name);
  else
    this.configuration.resetAnalyzer(block_entry.block.name);

  block_entry.algorithm = null;

  for (var i = 0; i < block_entry.block.nbInputs(); ++i)
    block_entry.block.inputs[i].dataformat = null;

  for (var i = 0; i < block_entry.block.nbOutputs(); ++i)
    block_entry.block.outputs[i].dataformat = null;

  // Update the description, parameters and datasets controls
  this._updateDescriptionControls(block_entry);
  this._updateDatasetsControls(block_entry);

  if (block_entry.parameters_panel)
  {
    block_entry.parameters_panel.destroy();
    block_entry.parameters_panel = null;
  }

  this._updateRunButton();

  if (this.onAlgorithmChanged)
    this.onAlgorithmChanged(null, previous_algorithm);
}


//----------------------------------------------------------------------
// Reset a dataset block (no dataset)
//----------------------------------------------------------------------
beat.experiments.panels.Settings.prototype._resetDataset = function(dataset_name)
{
  var selector = $('#' + this.id + ' select#' + dataset_name)[0];

  $(selector).find('option').remove();

  $('#' + this.id + ' #' + dataset_name + '__delete_button').hide();

  var first_option = $(document.createElement('option'));
  first_option.text('Select one dataset...');
  first_option.val('');
  first_option.disable();
  selector.appendChild(first_option[0]);

  $(selector).addClass('unselected');

  this.configuration.resetDataset(dataset_name);

  var dataset_block = this.toolchain.dataset(dataset_name);

  for (var j = 0; j < dataset_block.nbOutputs(); ++j)
    dataset_block.outputs[j].dataformat = null;

  for (var j = 0; j < selector.available_options.length; ++j) {
    var option = selector.available_options[j];

    var dataset = this.datasets.get(option.value);

    var result = dataset.isCompatible(dataset_block, this.dataformats);

    if (result.perfect) option.className = 'perfect';
    if (result.usable) selector.appendChild(option);
  }

  selector.selectedIndex = 0;
  $(selector).trigger("chosen:updated");
  this._updateRunButton();
}



/********************* CLASS: Parameters ***********************************/

//----------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------
beat.experiments.panels.Parameters = function(panel_id)
{
  this.id            = panel_id;
  this.element       = $('#' + panel_id)[0];
  this.configuration = null;
  this.algorithms    = null;
  this.environments  = null;
  this.sections      = {};
}


//----------------------------------------------------------------------
// Initialize the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype.initialize = function(configuration, algorithms,
    environments)
{
  this.configuration = configuration;
  this.algorithms    = algorithms;
  this.environments  = environments;

  $('#' + this.id + ' table').show();


  var environment_selector = $('#' + this.id + ' #environments')[0];
  var queue_selector       = $('#' + this.id + ' #queues')[0];

  var panel = this;

  $(environment_selector).change(function() {
    panel._onEnvironmentSelected();
  });

  $(queue_selector).change(function() {
    panel.configuration.setDefaultQueue(this.value);
  });


  var iterator = this.environments.iterator();
  var selected_environment_index = null;
  var first_python_environment_index = null;

  var default_environment = this.configuration.defaultEnvironment();
  if (default_environment !== null)
  {
    if (!this.environments.contains(default_environment.name, default_environment.version))
      default_environment = this.environments.get(default_environment.name);
  }

  var selector_counter = 0;
  while (iterator.hasMoreData())
  {
    var environment = iterator.next();

    var option         = document.createElement('option');
    option.textContent = environment.name + ' (' + environment.version + ')';
    option.value       = selector_counter;

    if ((default_environment != null) && (default_environment.name == environment.name) &&
      (default_environment.version == environment.version))
    {
      selected_environment_index = environment_selector.children.length;
    }

    if ((first_python_environment_index === null) && (environment.languages.indexOf('python') >= 0))
      first_python_environment_index = environment_selector.children.length;

    environment_selector.appendChild(option);
    selector_counter += 1;
  }

  if (selected_environment_index === null)
  {
    if (first_python_environment_index !== null)
      selected_environment_index = first_python_environment_index;
    else
      selected_environment_index = 0;
  }

  environment_selector.selectedIndex = selected_environment_index;
  this._onEnvironmentSelected();


  var selected_algorithms = this.configuration.algorithms();
  for (var i = 0; i < selected_algorithms.length; ++i)
  {
    var algorithm_name = selected_algorithms[i];
    this.addAlgorithm(selected_algorithms[i]);
  }
}


//----------------------------------------------------------------------
// Add an algorithm to the panel (only usable for a panel displaying global parameters)
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype.addAlgorithm = function(algorithm_name)
{
  var panel = this;

  // Controls already displayed: do nothing
  if (this.sections[algorithm_name] !== undefined)
    return;

  // Retrieve the infos about the algorithm
  var algorithm = this.algorithms.get(algorithm_name);
  if (!algorithm || !algorithm.parameters)
    return;

  // Add the controls
  var table = $('#' + this.id + ' table tbody')[0];
  var section = {
    header: null,
    lines: [],
  };

  section.header           = document.createElement('tr');
  section.header.className = 'active';
  table.appendChild(section.header);

  var td       = document.createElement('th');
  td.colSpan   = 2;
  td.className = 'action-buttons';
  section.header.appendChild(td);

  var span       = document.createElement('span');
  span.innerHTML = algorithm_name + ' ';
  td.appendChild(span);

  var reset_button = $('.algorithm-buttons>button:eq(0)').clone();
  reset_button.attr('title', 'Reset the parameters to their default values');
  reset_button.find('i').removeClass('fa-times').addClass('fa-undo');
  reset_button.css({display: 'inline'});
  reset_button = reset_button[0]; //extracts DOM object
  td.appendChild(reset_button);

  reset_button.controls = [];

  var global_parameters = this.configuration.algorithmGlobals(algorithm_name);
  for (var i = 0; i < algorithm.parameters.length; ++i)
  {
    var parameter = algorithm.parameters[i];

    if (global_parameters[parameter.name] === undefined)
      this.configuration.setAlgorithmGlobal(algorithm_name, parameter.name, parameter.default_value)

        this.createParameterControls(table, section, reset_button, algorithm,
            parameter, global_parameters,
            (i == algorithm.parameters.length - 1));
  }

  this.sections[algorithm_name] = section;


  $(reset_button).click(function() {
    for (var i = 0; i < this.controls.length; ++i)
    {
      var control = this.controls[i];

      if (control.type == 'checkbox')
      {
        control.checked = control.parameter.default_value;
        panel.configuration.setAlgorithmGlobal(control.algorithm, control.parameter.name, control.checked);
      }
      else
      {
        control.value = control.parameter.default_value;
        panel.configuration.setAlgorithmGlobal(control.algorithm, control.parameter.name, control.value);
      }
    }
  });
}


//----------------------------------------------------------------------
// Remove an algorithm from the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype.removeAlgorithm = function(algorithm_name)
{
  // Controls not displayed: do nothing
  var section = this.sections[algorithm_name];
  if (section === undefined)
    return;

  // Check that the algorithm is really not used anymore in the configuration
  var all_used_algorithms = this.configuration.algorithms();
  for (var i = 0; i < all_used_algorithms.length; ++i)
  {
    if (all_used_algorithms[i] == algorithm_name)
      return;
  }

  // Retrieve the infos about the algorithm
  var algorithm = this.algorithms.get(algorithm_name);
  if (!algorithm)
    return;

  // Remove the controls
  var table = $('#' + this.id + ' table tbody')[0];

  table.removeChild(section.header);
  for (var i = 0; i < section.lines.length; ++i)
    table.removeChild(section.lines[i]);

  delete this.sections[algorithm_name];
}


//----------------------------------------------------------------------
// Add the controls to change a parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype.createParameterControls = function(
    table, section, reset_button, algorithm,
    parameter, parameter_values, last_parameter)
{
  var panel = this;

  var tr       = document.createElement('tr');
  tr.className = 'parameter';
  table.appendChild(tr);

  if (last_parameter && !parameter.comment)
    tr.className += ' last';

  var td       = document.createElement('th');
  td.innerHTML = parameter.name;
  tr.appendChild(td);

  td           = document.createElement('td');
  tr.appendChild(td);

  if ((parameter.type == 'uint8') || (parameter.type == 'uint16') ||
      (parameter.type == 'uint32') || (parameter.type == 'uint64') ||
      (parameter.type == 'int8') || (parameter.type == 'int16') ||
      (parameter.type == 'int32') || (parameter.type == 'int64'))
  {
    this._createIntegerControls(td, reset_button, algorithm, parameter, parameter_values);
  }
  else if ((parameter.type == 'float32') || (parameter.type == 'float64'))
  {
    this._createFloatControls(td, reset_button, algorithm, parameter, parameter_values);
  }
  else if (parameter.type == 'bool')
  {
    this._createBoolControls(td, reset_button, algorithm, parameter, parameter_values);
  }
  else if (parameter.type == 'string')
  {
    this._createStringControls(td, reset_button, algorithm, parameter, parameter_values);
  }

  section.lines.push(tr);

  if (parameter.comment)
  {
    var help = $(td).find('span.help');
    if (help) {
      help.text(help.text() + ' ' + parameter.comment);
    }
    else {
      var tr = document.createElement('tr');
      tr.className = 'help';
      table.appendChild(tr);
      if (last_parameter) tr.className += ' last';
      var td = document.createElement('td');
      tr.appendChild(td);
      td = document.createElement('td');
      td.innerHTML = parameter.comment;
      tr.appendChild(td);
      section.lines.push(tr);
    }
  }
}


//----------------------------------------------------------------------
// Add the controls to change an integer parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype._createIntegerControls = function(
    cell, reset_button, algorithm, parameter,
    parameter_values)
{
  var panel = this;

  // Range or any value
  if (parameter.choices === undefined)
  {
    var input = document.createElement('input');
    input.className = 'form-control input-sm';
    cell.appendChild(input);

    if (parameter_values[parameter.name] !== undefined)
      input.value = parameter_values[parameter.name];
    else
      input.value = parameter.default_value;

    input.algorithm = algorithm.name;
    input.parameter = parameter;

    reset_button.controls.push(input);

    var span       = document.createElement('span');
    span.className = 'help';
    cell.appendChild(span);

    if (parameter.minimum !== undefined)
      span.innerHTML = '(' + parameter.type + ': [' + parameter.minimum + ', ' + parameter.maximum + '])';
    else
      span.innerHTML = '(' + parameter.type + ')';

    $(input).change(function() {
      var value = parseInt(this.value);
      if (isNaN(value))
      {
        this.value = panel.configuration.algorithmGlobals(this.algorithm)[this.parameter.name];
        return;
      }

      var minimum = null;
      var maximum = null;

      if (this.parameter.minimum !== undefined)
      {
        minimum = this.parameter.minimum;
        maximum = this.parameter.maximum;
      }
      else if (input.parameter.type == 'uint8')
      {
        minimum = 0;
        maximum = 255;
      }
      else if (input.parameter.type == 'uint16')
      {
        minimum = 0;
        maximum = Math.pow(2, 16) - 1;
      }
      else if (input.parameter.type == 'uint32')
      {
        minimum = 0;
        maximum = Math.pow(2, 32) - 1;
      }
      else if (input.parameter.type == 'uint64')
      {
        minimum = 0;
        maximum = Math.pow(2, 64) - 1;
      }
      else if (input.parameter.type == 'int8')
      {
        maximum = 127;
        minimum = -128;
      }
      else if (input.parameter.type == 'int16')
      {
        maximum = Math.pow(2, 15 - 1);
        minimum = -Math.pow(2, 15);
      }
      else if (input.parameter.type == 'int32')
      {
        maximum = Math.pow(2, 31 - 1);
        minimum = -Math.pow(2, 31);
      }
      else if (input.parameter.type == 'int64')
      {
        maximum = Math.pow(2, 63 - 1);
        minimum = -Math.pow(2, 63);
      }

      if (value < minimum)
      {
        value = minimum;
        this.value = minimum;
      }
      else if (value > maximum)
      {
        value = maximum;
        this.value = maximum;
      }

      panel.configuration.setAlgorithmGlobal(this.algorithm, this.parameter.name, value);
    });
  }

  // Choices
  else
  {
    var select = document.createElement('select');
    select.className = 'form-control input-sm';
    cell.appendChild(select);

    select.algorithm = algorithm.name;
    select.parameter = parameter;

    reset_button.controls.push(select);

    var span       = document.createElement('span');
    span.className = 'help';
    span.innerHTML = '(' + parameter.type + ')';
    cell.appendChild(span);

    var value = (parameter_values[parameter.name] !== undefined ? parameter_values[parameter.name] : parameter.default_value);

    for (var i = 0; i < parameter.choices.length; ++i)
    {
      var option   = document.createElement('option');
      option.text  = parameter.choices[i];
      option.value = parameter.choices[i];
      select.appendChild(option);

      if (option.value == value)
        select.selectedIndex = i;
    }

    $(select).change(function() {
      panel.configuration.setAlgorithmGlobal(this.algorithm, this.parameter.name, this.value);
    });
  }
}


//----------------------------------------------------------------------
// Add the controls to change a floating-point parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype._createFloatControls = function(
    cell, reset_button, algorithm, parameter,
    parameter_values)
{
  var panel = this;

  // Range or any value
  if (parameter.choices === undefined)
  {
    var input = document.createElement('input');
    input.className = 'form-control input-sm';
    cell.appendChild(input);

    if (parameter_values[parameter.name] !== undefined)
      input.value = parameter_values[parameter.name];
    else
      input.value = parameter.default_value;

    input.algorithm = algorithm.name;
    input.parameter = parameter;

    reset_button.controls.push(input);

    var span       = document.createElement('span');
    span.className = 'help';
    cell.appendChild(span);

    if (parameter.minimum !== undefined)
      span.innerHTML = '(' + parameter.type + ': [' + parameter.minimum + ', ' + parameter.maximum + '])';
    else
      span.innerHTML = '(' + parameter.type + ')';

    $(input).change(function() {
      var value = parseFloat(this.value);
      if (isNaN(value))
      {
        this.value = panel.configuration.algorithmGlobals(this.algorithm)[this.parameter.name];
        return;
      }

      var minimum = null;
      var maximum = null;

      if (this.parameter.minimum !== undefined)
      {
        if (value < this.parameter.minimum)
        {
          value = this.parameter.minimum;
          this.value = this.parameter.minimum;
        }
        else if (value > this.parameter.maximum)
        {
          value = this.parameter.maximum;
          this.value = this.parameter.maximum;
        }
      }

      panel.configuration.setAlgorithmGlobal(this.algorithm, this.parameter.name, value);
    });
  }

  // Choices
  else
  {
    var select = document.createElement('select');
    select.className = 'form-control input-sm';
    cell.appendChild(select);

    select.algorithm = algorithm.name;
    select.parameter = parameter;

    reset_button.controls.push(select);

    var value = (parameter_values[parameter.name] !== undefined ? parameter_values[parameter.name] : parameter.default_value);

    for (var i = 0; i < parameter.choices.length; ++i)
    {
      var option   = document.createElement('option');
      option.text  = parameter.choices[i];
      option.value = parameter.choices[i];
      select.appendChild(option);

      if (option.value == value)
        select.selectedIndex = i;
    }

    $(select).change(function() {
      panel.configuration.setAlgorithmGlobal(this.algorithm, this.parameter.name, this.value);
    });
  }
}


//----------------------------------------------------------------------
// Add the controls to change a boolean parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype._createBoolControls = function(
    cell, reset_button, algorithm, parameter,
    parameter_values)
{
  var panel = this;

  var div = document.createElement('div');
  div.className = 'checkbox';
  cell.appendChild(div);

  var input       = document.createElement('input');
  input.type      = 'checkbox';
  cell.appendChild(input);

  input.algorithm = algorithm.name;
  input.parameter = parameter;

  reset_button.controls.push(input);

  var span       = document.createElement('span');
  span.className = 'help';
  span.innerHTML = '(' + parameter.type + ')';
  cell.appendChild(span);

  input.checked = (parameter_values[parameter.name] !== undefined ? parameter_values[parameter.name] : parameter.default_value);

  $(input).change(function() {
    panel.configuration.setAlgorithmGlobal(this.algorithm, this.parameter.name, this.checked);
  });
}


//----------------------------------------------------------------------
// Add the controls to change a string parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype._createStringControls = function(
    cell, reset_button, algorithm, parameter,
    parameter_values)
{
  var panel = this;

  var select = document.createElement('select');
  select.className = 'form-control input-sm';
  cell.appendChild(select);

  select.algorithm = algorithm.name;
  select.parameter = parameter;

  reset_button.controls.push(select);

  var span       = document.createElement('span');
  span.className = 'help';
  span.innerHTML = '(' + parameter.type + ')';
  cell.appendChild(span);

  var value = (parameter_values[parameter.name] !== undefined ? parameter_values[parameter.name] : parameter.default_value);

  for (var i = 0; i < parameter.choices.length; ++i)
  {
    var option   = document.createElement('option');
    option.text  = parameter.choices[i];
    option.value = parameter.choices[i];
    select.appendChild(option);

    if (option.value == value)
      select.selectedIndex = i;
  }

  $(select).change(function() {
    panel.configuration.setAlgorithmGlobal(this.algorithm, this.parameter.name, this.value);
  });
}


//----------------------------------------------------------------------
// Called when the default environment is selected
//----------------------------------------------------------------------
beat.experiments.panels.Parameters.prototype._onEnvironmentSelected = function()
{
  var environment_selector = $('#' + this.id + ' #environments')[0];
  var queue_selector = $('#' + this.id + ' #queues')[0];

  $(queue_selector).empty();

  var environment = this.environments.getByIndex(environment_selector.value);

  var config_env = {
    name: environment.name,
    version: environment.version,
  };

  this.configuration.setDefaultEnvironment(config_env);

  var selected_queue_index = null;
  var default_queue = this.configuration.defaultQueue();
  for (var i = 0; i < environment.queues.length; ++i)
  {
    var queue = environment.queues[i];

    var option         = document.createElement('option');
    option.textContent = queue.name + ' (' + queue.memory_limit + ' MB / ' + queue.time_limit + '\' / ' + queue.nb_cores_per_slot
        if (queue.nb_cores_per_slot == 1)
        option.textContent += ' core';
        else
        option.textContent += ' cores';
        option.textContent += ' per slot / ' + queue.nb_slots;
        if (queue.nb_slots == 1)
        option.textContent += ' slot';
        else
        option.textContent += ' slots';
        option.textContent += ' / ' + queue.max_slots_per_user;
        if (queue.max_slots_per_user == 1)
        option.textContent += ' slot';
        else
        option.textContent += ' slots';
        option.textContent += ' max per user)';
    option.value       = queue.name;

    if (default_queue == queue.name)
      selected_queue_index = queue_selector.children.length;

    queue_selector.appendChild(option);
  }

  if (selected_queue_index === null)
    selected_queue_index = 0;

  queue_selector.selectedIndex = selected_queue_index;
  this.configuration.setDefaultQueue(queue_selector.value);
}



/****************** CLASS: ParametersViewer ********************************/

//----------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------
beat.experiments.panels.ParametersViewer = function(url_prefix, panel_id)
{
  this.url_prefix    = url_prefix;
  this.id            = panel_id;
  this.element       = $('#' + panel_id)[0];
  this.configuration = null;
}


//----------------------------------------------------------------------
// Initialize the panel
//----------------------------------------------------------------------
beat.experiments.panels.ParametersViewer.prototype.initialize = function(configuration)
{
  this.configuration = configuration;

  // Add the controls
  var table = $('#' + this.id + ' table tbody')[0];


  // Default environment
  var tr       = document.createElement('tr');
  tr.className = 'parameter';
  table.appendChild(tr);

  var td       = document.createElement('th');
  td.innerHTML = 'Default environment:';
  tr.appendChild(td);

  var default_environment = this.configuration.defaultEnvironment();

  td           = document.createElement('td');
  tr.appendChild(td);

  var a         = document.createElement('a');
  a.href        = this.url_prefix + '/backend/environments/' + default_environment.name + '/' + default_environment.version + '/';
  a.textContent = default_environment.name + ' (' + default_environment.version + ')';
      td.appendChild(a);


      // Default queue
      tr           = document.createElement('tr');
      tr.className = 'parameter';
      table.appendChild(tr);

      td           = document.createElement('th');
      td.innerHTML = 'Default queue:';
      tr.appendChild(td);

      var default_queue = this.configuration.defaultQueue();

      td           = document.createElement('td');
      td.innerHTML = default_queue;
      tr.appendChild(td);


      var block_names = [].concat(this.configuration.blockNames(), this.configuration.analyzerNames());

      var selected_algorithms = this.configuration.algorithms();
      for (var i = 0; i < selected_algorithms.length; ++i)
      {
        var algorithm_name = selected_algorithms[i];

        var global_parameters = JSON.parse(JSON.stringify(this.configuration.algorithmGlobals(algorithm_name)));

        var parameter_names = Object.keys(global_parameters);
        if (parameter_names.length == 0)
          continue;

        var blocks = [];
        for (var j = 0; j < block_names.length; ++j)
        {
          var block_configuration = this.configuration.component(block_names[j]);
          if (block_configuration.algorithm == algorithm_name)
            blocks.push(block_configuration);
        }

        for (var j = 0; j < parameter_names.length; ++j)
        {
          var parameter_name = parameter_names[j];

          var needed = false;
          for (var k = 0; k < blocks.length; ++k)
          {
            if ((blocks[k].parameters === undefined) || (blocks[k].parameters === null) ||
                (blocks[k].parameters[parameter_name] === undefined))
            {
              needed = true;
              break;
            }
          }

          if (!needed)
            delete global_parameters[parameter_name];
        }

        var parameter_names = Object.keys(global_parameters);
        if (parameter_names.length == 0)
          continue;

        var header       = document.createElement('tr');
        header.className = 'active';
        table.appendChild(header);

        var td     = document.createElement('td');
        td.colSpan = 2;
        header.appendChild(td);

        var span       = document.createElement('span');
        span.innerHTML = algorithm_name;
        td.appendChild(span);

        for (var j = 0; j < parameter_names.length; ++j)
        {
          var tr       = document.createElement('tr');
          tr.className = 'parameter';
          table.appendChild(tr);

          if (j == parameter_names.length - 1)
            tr.className += ' last';

          var td       = document.createElement('th');
          td.innerHTML = parameter_names[j];
          tr.appendChild(td);

          td           = document.createElement('td');
          td.innerHTML = global_parameters[parameter_names[j]];
          tr.appendChild(td);
        }
      }

      for (var i = 0; i < block_names.length; ++i)
      {
        var block_name = block_names[i];

        var component_configuration = this.configuration.component(block_name);
        var component_queue = this.configuration.blockQueue(block_name);
        var component_environment = this.configuration.blockEnvironment(block_name);
        var component_nb_slots = this.configuration.blockNbSlots(block_name);

        var parameters = component_configuration.parameters;
        var parameter_names = null;

        if (parameters !== undefined)
          parameter_names = Object.keys(parameters);

        var environment = null;
        if ((component_environment != null) &&
            ((component_environment.name != default_environment.name) ||
             (component_environment.version != default_environment.version)))
        {
          environment = component_environment;
        }

        var queue = null;
        if ((component_queue != null) && (component_queue != default_queue))
          queue = component_queue;

        var nb_slots = null;
        if ((component_nb_slots != null) && (component_nb_slots != 1))
          nb_slots = component_nb_slots;

        if (((parameters === undefined) || (parameter_names.length == 0)) &&
            (environment == null) && (queue == null) && (nb_slots == null))
          continue;

        var header       = document.createElement('tr');
        header.className = 'active';
        table.appendChild(header);

        var td     = document.createElement('td');
        td.colSpan = 2;
        header.appendChild(td);

        var span       = document.createElement('span');
        span.innerHTML = 'Block: ' + block_name;
        td.appendChild(span);

        if (environment != null)
        {
          var tr       = document.createElement('tr');
          tr.className = 'parameter';
          table.appendChild(tr);

          var td       = document.createElement('th');
          td.innerHTML = 'environment:';
          tr.appendChild(td);

          td           = document.createElement('td');
          tr.appendChild(td);

          var a         = document.createElement('a');
          a.href        = this.url_prefix + '/backend/environments/' + environment.name + '/' + environment.version + '/';
          a.textContent = environment.name + ' (' + environment.version + ')';
              td.appendChild(a);
              }


              if (queue != null)
              {
                var tr       = document.createElement('tr');
                tr.className = 'parameter';
                table.appendChild(tr);

                var td       = document.createElement('th');
                td.innerHTML = 'queue:';
                tr.appendChild(td);

                td           = document.createElement('td');
                td.innerHTML = queue;
                tr.appendChild(td);
              }


          if (nb_slots != null)
          {
            var tr       = document.createElement('tr');
            tr.className = 'parameter';
            table.appendChild(tr);

            var td       = document.createElement('th');
            td.innerHTML = 'nb slots:';
            tr.appendChild(td);

            td           = document.createElement('td');
            td.innerHTML = nb_slots;
            tr.appendChild(td);
          }


          if (parameter_names != null)
          {
            for (var j = 0; j < parameter_names.length; ++j)
            {
              var tr       = document.createElement('tr');
              tr.className = 'parameter';
              table.appendChild(tr);

              if (j == parameter_names.length - 1)
                tr.className += ' last';

              var td       = document.createElement('th');
              td.innerHTML = parameter_names[j];
              tr.appendChild(td);

              td           = document.createElement('td');
              td.innerHTML = parameters[parameter_names[j]];
              tr.appendChild(td);
            }
          }
      }
}



/******************* CLASS: ExecutionInfos *********************************/

//----------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------
beat.experiments.panels.ExecutionInfos = function(panel_id)
{
  this.id               = panel_id;
  this.element          = $('#' + panel_id)[0];
  this.blocks_displayed = [];
}


//----------------------------------------------------------------------
// Update the content of the panel
//----------------------------------------------------------------------
beat.experiments.panels.ExecutionInfos.prototype.update = function(
    execution_infos, experiment_finished)
{
  function _addValue(table, name, value, tooltip)
  {
    var tr       = document.createElement('tr');
    tr.className = 'parameter';
    table.appendChild(tr);

    if (tooltip !== undefined)
      tr.title = tooltip;

    var td       = document.createElement('th');
    td.innerHTML = name;
    tr.appendChild(td);

    td           = document.createElement('td');
    td.innerHTML = value;
    tr.appendChild(td);
  }

  function _secondsToTime(secs)
  {
    if (secs < 60.)
    {
      return secs.toFixed(2) + ' seconds';
    }
    else if (secs < 3600.)
    {
      return (secs/60.).toFixed(2) + ' minutes';
    }
    else if (secs < (3600.*24))
    {
      return  (secs/3600.).toFixed(2) + ' hours';
    }
    else
    {
      return  (secs/(3600.*24)).toFixed(2) + ' days';
    }
  }

  if (experiment_finished)
    $('#' + this.id + ' div.progress').hide();


  // Add the new infos
  var table = $('#' + this.id + ' table tbody')[0];
  var block_names = Object.keys(execution_infos);
  block_names.sort();

  for (var i = 0; i < block_names.length; ++i)
  {
    var block_name = block_names[i];

    if (this.blocks_displayed.indexOf(block_name) >= 0)
      continue;

    var execution_info = execution_infos[block_name];

    var header       = document.createElement('tr');
    header.className = 'active';
    table.appendChild(header);

    var td     = document.createElement('td');
    td.colSpan = 2;
    header.appendChild(td);

    var span       = document.createElement('span');
    span.innerHTML = 'Block: ' + block_name;
    td.appendChild(span);

    _addValue(table, 'queuing time', _secondsToTime(execution_info.queuing_time));
    _addValue(table, 'sequential execution time', _secondsToTime(execution_info.linear_execution_time));

    if (execution_info.speed_up_maximal > 1)
    {
      _addValue(table, 'speed-up', execution_info.speed_up_real.toFixed(2) + 'x / ' +
          execution_info.speed_up_maximal.toFixed(2) + 'x',
          'real speed-up / maximal theoretical speed-up');
    }

    this.blocks_displayed.push(block_name);
  }
}



/******************* CLASS: BlockParameters ********************************/

//----------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters = function(first_row, configuration, block_name,
    algorithm, environments)
{
  this.configuration = configuration;
  this.algorithm     = algorithm;
  this.block_name    = block_name;
  this.environments  = environments;
  this.lines         = [];


  var has_parameters = (algorithm.parameters != null) && (algorithm.parameters.length > 0);


  // Environment & queues
  this.createEnvironmentControls(first_row, block_name, !has_parameters);


  // Parameters
  if (has_parameters)
  {
    // Retrieve the current parameter values
    var parameters = this.configuration.componentParameters(block_name);
    if (!parameters)
      return;

    // Retrieve the global parameter values
    var global_parameters = this.configuration.algorithmGlobals(algorithm.name);
    if (!global_parameters)
      return;

    // Cleanup
    var valid_parameters = algorithm.parameters.map(function(x) { return x.name; });

    var block_configuration = this.configuration.component(block_name);

    var set_parameters = Object.keys(parameters);
    for (var i = 0; i < set_parameters.length; ++i)
    {
      if (valid_parameters.indexOf(set_parameters[i]) == -1)
      {
        delete parameters[set_parameters[i]];
        delete block_configuration.parameters[set_parameters[i]];
      }
    }

    // Add the controls
    for (var i = 0; i < algorithm.parameters.length; ++i)
    {
      if (parameters[algorithm.parameters[i].name] === undefined)
        parameters[algorithm.parameters[i].name] = algorithm.parameters[i].default_value;

      if (global_parameters[algorithm.parameters[i].name] === undefined)
        global_parameters[algorithm.parameters[i].name] = algorithm.parameters[i].default_value;

      this.createParameterControls(this.lines[this.lines.length - 1],
          algorithm.parameters[i], parameters, global_parameters,
          (i == algorithm.parameters.length - 1),
          (i == 0));
    }
  }
}


//----------------------------------------------------------------------
// Add the controls to change a parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype.createEnvironmentControls = function(
    predecessor, block_name, last)
{
  var panel = this;


  // Environments
  var tr       = document.createElement('tr');
  tr.className = 'parameter';
  $(tr).insertAfter(predecessor);

  this.lines.push(tr);

  var td       = document.createElement('th');
  td.innerHTML = 'Environment';
  tr.appendChild(td);

  td           = document.createElement('td');
  td.colSpan   = 2;
  tr.appendChild(td);

  var environment_selector       = document.createElement('select');
  environment_selector.className = 'form-control input-sm environment';
  td.appendChild(environment_selector);


  // Queues
  tr           = document.createElement('tr');
  tr.className = 'parameter';
  $(tr).insertAfter(this.lines[0]);

  this.lines.push(tr);

  td           = document.createElement('th');
  td.innerHTML = 'Queue:';
  tr.appendChild(td);

  td           = document.createElement('td');
  td.colSpan   = 2;
  tr.appendChild(td);

  var queue_selector       = document.createElement('select');
  queue_selector.className = 'form-control input-sm environment';
  td.appendChild(queue_selector);


  // Nb slots
  var slots_line = document.createElement('tr');
  slots_line.className   = 'parameter';
  $(slots_line).insertAfter(this.lines[1]);

  this.lines.push(slots_line);

  td           = document.createElement('th');
  td.innerHTML = 'Number of slots:';
  slots_line.appendChild(td);

  td           = document.createElement('td');
  td.colSpan   = 2;
  slots_line.appendChild(td);

  var slots_input = document.createElement('input');
  slots_input.className = 'form-control input-sm';
  td.appendChild(slots_input);


  function _onNbSlotsChanged()
  {
    var value = parseInt(slots_input.value);
    if (isNaN(value))
    {
      value = 1;
      slots_input.value = value;
    }

    var environment = panel.environments.getByIndex(environment_selector.value);

    if (value < 1)
    {
      value = 1;
      slots_input.value = 1;
    }

    panel.configuration.setBlockNbSlots(panel.block_name, value);

    if (value > 1)
      $(slots_input).removeClass('default');
    else
      $(slots_input).addClass('default');
  }


  function _onQueueChanged()
  {
    panel.configuration.setBlockQueue(block_name, queue_selector.value);
    slots_input.value = panel.configuration.blockNbSlots(block_name);


    if (queue_selector.value == panel.configuration.defaultQueue())
      $(queue_selector).addClass('default');
    else
      $(queue_selector).removeClass('default');


    var environment = panel.environments.getByIndex(environment_selector.value);

    for (var i = 0; i < environment.queues.length; ++i)
    {
      var queue = environment.queues[i];

      if (queue.name == queue_selector.value)
      {
        if (panel.algorithm.splittable)
        {
          $(slots_line).show();
        }
        else
        {
          $(slots_line).hide();
        }

        _onNbSlotsChanged();

        break;
      }
    }
  }


  function _onEnvironmentChanged()
  {
    $(queue_selector).empty();

    var environment = panel.environments.getByIndex(environment_selector.value);

    var config_env = {
      name: environment.name,
      version: environment.version,
    };
    panel.configuration.setBlockEnvironment(block_name, config_env);

    var default_environment = panel.configuration.defaultEnvironment();

    if (environment.name == default_environment.name && environment.version == default_environment.version)
      $(environment_selector).addClass('default');
    else
      $(environment_selector).removeClass('default');

    var selected_queue_index = null;
    for (var i = 0; i < environment.queues.length; ++i)
    {
      var queue = environment.queues[i];

      var option         = document.createElement('option');
      option.textContent = queue.name + ' (' + queue.memory_limit + ' MB / ' + queue.time_limit + '\' / ' + queue.nb_cores_per_slot
          if (queue.nb_cores_per_slot == 1)
          option.textContent += ' core';
          else
          option.textContent += ' cores';
          option.textContent += ' per slot / ' + queue.nb_slots;
          if (queue.nb_slots == 1)
          option.textContent += ' slot';
          else
          option.textContent += ' slots';
          option.textContent += ' / ' + queue.max_slots_per_user;
          if (queue.max_slots_per_user == 1)
          option.textContent += ' slot';
          else
          option.textContent += ' slots';
          option.textContent += ' max per user)';
      option.value       = queue.name;

      if (queue.name == panel.configuration.defaultQueue())
        $(option).addClass('default');

      if (panel.configuration.blockQueue(block_name) == queue.name)
        selected_queue_index = queue_selector.children.length;

      queue_selector.appendChild(option);
    }

    if (selected_queue_index === null)
      selected_queue_index = 0;

    queue_selector.selectedIndex = selected_queue_index;

    _onQueueChanged();
  }


  $(environment_selector).change(_onEnvironmentChanged);
  $(queue_selector).change(_onQueueChanged);
  $(slots_input).change(_onNbSlotsChanged);


  this.configuration.addDefaultEnvironmentListener(this.block_name, function(previous_value, new_value) {
    var selected_environment = panel.environments.getByIndex(environment_selector.value);

    var can_be_changed = (selected_environment.name == previous_value.name) &&
      (selected_environment.version == previous_value.version);

    for (var i = 0; i < environment_selector.options.length; ++i)
    {
      var environment = panel.environments.getByIndex(i);

      if ((environment.name == new_value.name) && (environment.version == new_value.version))
      {
        $(environment_selector.options[i]).addClass('default');

        if (can_be_changed)
          environment_selector.selectedIndex = i;
      }
      else
      {
        environment_selector.options[i].className = '';
      }
    }

    if (!can_be_changed && (selected_environment.name == new_value.name) &&
        (selected_environment.version == new_value.version))
    {
      $(environment_selector).addClass('default');
    }

    _onEnvironmentChanged();
  });


  this.configuration.addDefaultQueueListener(this.block_name, function(previous_value, new_value) {
    var can_be_changed = (queue_selector.value == previous_value);

    for (var i = 0; i < queue_selector.options.length; ++i)
    {
      if (queue_selector.options[i].value == new_value)
      {
        $(queue_selector.options[i].className).addClass('default');

        if (can_be_changed)
          queue_selector.selectedIndex = i;
      }
      else
      {
        queue_selector.options[i].className = '';
      }
    }

    if (!can_be_changed && (queue_selector.value == new_value))
      $(queue_selector).addClass('default');
  });


  // Populate the environments selector
  var iterator = this.environments.iterator();
  var selected_environment_index = null;

  var selected_environment = this.configuration.blockEnvironment(block_name);
  var default_environment = panel.configuration.defaultEnvironment();
  var selector_counter = 0;
  while (iterator.hasMoreData())
  {
    var environment = iterator.next();

    var option         = document.createElement('option');
    option.textContent = environment.name + ' (' + environment.version + ')';
        option.value       = selector_counter;

        if (environment.name == default_environment.name && environment.version == default_environment.version)
        $(option).addClass('default');

        if (selected_environment.name == environment.name && selected_environment.version == environment.version)
        selected_environment_index = environment_selector.children.length;

        environment_selector.appendChild(option);
        selector_counter += 1;
        }

        environment_selector.selectedIndex = selected_environment_index;

        _onEnvironmentChanged();
        }


        //----------------------------------------------------------------------
        // Add the controls to change a parameter value to the panel
        //----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype.createParameterControls = function(
    predecessor, parameter, parameter_values,
    global_parameters, last, first)
{
  var tr       = document.createElement('tr');
  tr.className = 'parameter';
  $(tr).insertAfter(predecessor);

  if (first)
    $(tr).addClass('first');

  var td       = document.createElement('th');
  td.innerHTML = parameter.name;
  tr.appendChild(td);

  td           = document.createElement('td');
  td.colSpan   = 2;
  tr.appendChild(td);

  if ((parameter.type == 'uint8') || (parameter.type == 'uint16') ||
      (parameter.type == 'uint32') || (parameter.type == 'uint64') ||
      (parameter.type == 'int8') || (parameter.type == 'int16') ||
      (parameter.type == 'int32') || (parameter.type == 'int64'))
  {
    this._createIntegerControls(td, parameter, parameter_values, global_parameters);
  }
  else if ((parameter.type == 'float32') || (parameter.type == 'float64'))
  {
    this._createFloatControls(td, parameter, parameter_values, global_parameters);
  }
  else if (parameter.type == 'bool')
  {
    this._createBoolControls(td, parameter, parameter_values, global_parameters);
  }
  else if (parameter.type == 'string')
  {
    this._createStringControls(td, parameter, parameter_values, global_parameters);
  }

  this.lines.push(tr);

  if (parameter.comment)
  {
    var help = $(td).find('span.help');
    if (help) {
      help.text(help.text() + ' ' + parameter.comment);
    }
    else {
      var tr2       = document.createElement('tr');
      tr2.className = 'help';
      var td = document.createElement('td');
      tr2.appendChild(td);
      td           = document.createElement('td');
      td.innerHTML = parameter.comment;
      tr2.appendChild(td);
      this.lines.push(tr2);
      $(tr2).insertAfter(tr);
    }
  }

  if (last)
  {
    var last_row = this.lines[this.lines.length - 1];
    $(last_row).addClass('last');
  }
}


//----------------------------------------------------------------------
// Remove all the controls of the panel
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype.destroy = function()
{
  this.configuration.removeDefaultEnvironmentListener(this.block_name);
  this.configuration.removeDefaultQueueListener(this.block_name);

  if (this.algorithm.parameters)
  {
    for (var i = 0; i < this.algorithm.parameters.length; ++i)
    {
      this.configuration.removeGlobalParameterListener(this.algorithm.name,
          this.algorithm.parameters[i].name,
          this.block_name);
    }
  }

  for (var i = 0; i < this.lines.length; ++i)
  {
    var line = this.lines[i];
    line.parentElement.removeChild(line);
  }
}


//----------------------------------------------------------------------
// Add the controls to change an integer parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype._createIntegerControls = function(
    cell, parameter, parameter_values, global_parameters)
{
  var panel = this;

  // Range or any value
  if (parameter.choices === undefined)
  {
    var input = document.createElement('input');
    input.className = 'form-control input-sm';
    cell.appendChild(input);

    input.value = parameter_values[parameter.name];

    if (input.value == global_parameters[parameter.name])
      $(input).addClass('default');

    input.block_name = this.block_name;
    input.parameter = parameter;

    var span       = document.createElement('span');
    span.className = 'help';
    cell.appendChild(span);

    if (parameter.minimum !== undefined)
      span.innerHTML = '(' + parameter.type + ': [' + parameter.minimum + ', ' + parameter.maximum + '])';
    else
      span.innerHTML = '(' + parameter.type + ')';

    $(input).change(function() {
      var value = parseInt(this.value);
      if (isNaN(value))
      {
        var parameters = panel.configuration.componentParameters(panel.block_name);
        this.value = parameters[this.parameter.name];

        if (this.value != panel.configuration.algorithmGlobals(panel.algorithm.name)[this.parameter.name])
          $(this).removeClass('default');
        else
          $(this).addClass('default');

        return;
      }

      var minimum = null;
      var maximum = null;

      if (this.parameter.minimum !== undefined)
      {
        minimum = this.parameter.minimum;
        maximum = this.parameter.maximum;
      }
      else if (input.parameter.type == 'uint8')
      {
        minimum = 0;
        maximum = 255;
      }
      else if (input.parameter.type == 'uint16')
      {
        minimum = 0;
        maximum = Math.pow(2, 16) - 1;
      }
      else if (input.parameter.type == 'uint32')
      {
        minimum = 0;
        maximum = Math.pow(2, 32) - 1;
      }
      else if (input.parameter.type == 'uint64')
      {
        minimum = 0;
        maximum = Math.pow(2, 64) - 1;
      }
      else if (input.parameter.type == 'int8')
      {
        maximum = 127;
        minimum = -128;
      }
      else if (input.parameter.type == 'int16')
      {
        maximum = Math.pow(2, 15 - 1);
        minimum = -Math.pow(2, 15);
      }
      else if (input.parameter.type == 'int32')
      {
        maximum = Math.pow(2, 31 - 1);
        minimum = -Math.pow(2, 31);
      }
      else if (input.parameter.type == 'int64')
      {
        maximum = Math.pow(2, 63 - 1);
        minimum = -Math.pow(2, 63);
      }

      if (value < minimum)
      {
        value = minimum;
        this.value = minimum;
      }
      else if (value > maximum)
      {
        value = maximum;
        this.value = maximum;
      }

      if (panel.configuration.setComponentParameter(panel.block_name, this.parameter.name, value))
        $(this).removeClass('default');
      else
        $(this).addClass('default');
    });

    this.configuration.addGlobalParameterListener(this.algorithm.name, parameter.name,
        this.block_name, function(previous_value, new_value) {
          if (input.value == previous_value)
            input.value = new_value;
        });
  }

  // Choices
  else
  {
    var select = document.createElement('select');
    select.className = 'form-control input-sm';
    cell.appendChild(select);

    select.block_name = this.block_name;
    select.parameter = parameter;

    var span       = document.createElement('span');
    span.className = 'help';
    span.innerHTML = '(' + parameter.type + ')';
    cell.appendChild(span);

    var value = parameter_values[parameter.name];

    for (var i = 0; i < parameter.choices.length; ++i)
    {
      var option   = document.createElement('option');
      option.text  = parameter.choices[i];
      option.value = parameter.choices[i];
      select.appendChild(option);

      if (option.value == value)
        select.selectedIndex = i;

      if (option.value == global_parameters[parameter.name])
      {
        $(option).addClass('default');

        if (option.value == value)
          $(select).addClass('default');
      }
    }

    $(select).change(function() {
      if (panel.configuration.setComponentParameter(panel.block_name, this.parameter.name, this.value))
        $(this).removeClass('default');
      else
        $(this).addClass('default');
    });

    this.configuration.addGlobalParameterListener(this.algorithm.name, parameter.name,
        this.block_name, function(previous_value, new_value) {
          var can_be_changed = (select.value == previous_value);

          for (var i = 0; i < select.options.length; ++i)
          {
            if (select.options[i].value == new_value)
            {
              $(select.options[i]).addClass('default');

              if (can_be_changed)
                select.selectedIndex = i;
            }
            else
            {
              select.options[i].className = '';
            }
          }

          if (!can_be_changed && (select.value == new_value))
            $(select).addClass('default');
        });
  }
}


//----------------------------------------------------------------------
// Add the controls to change a floating-point parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype._createFloatControls = function(
    cell, parameter, parameter_values, global_parameters)
{
  var panel = this;

  // Range or any value
  if (parameter.choices === undefined)
  {
    var input = document.createElement('input');
    input.className = 'form-control input-sm';
    cell.appendChild(input);

    input.value = parameter_values[parameter.name];

    if (input.value == global_parameters[parameter.name])
      $(input).addClass('default');

    input.block_name = this.block_name;
    input.parameter = parameter;

    var span       = document.createElement('span');
    span.className = 'help';
    cell.appendChild(span);

    if (parameter.minimum !== undefined)
      span.innerHTML = '(' + parameter.type + ': [' + parameter.minimum + ', ' + parameter.maximum + '])';
    else
      span.innerHTML = '(' + parameter.type + ')';

    $(input).change(function() {
      var value = parseFloat(this.value);
      if (isNaN(value))
      {
        var parameters = panel.configuration.componentParameters(panel.block_name);
        this.value = parameters[this.parameter.name];

        if (this.value != panel.configuration.algorithmGlobals(panel.algorithm.name)[this.parameter.name])
          $(this).removeClass('default');
        else
          $(this).addClass('default');

        return;
      }

      var minimum = null;
      var maximum = null;

      if (this.parameter.minimum !== undefined)
      {
        if (value < this.parameter.minimum)
        {
          value = this.parameter.minimum;
          this.value = this.parameter.minimum;
        }
        else if (value > this.parameter.maximum)
        {
          value = this.parameter.maximum;
          this.value = this.parameter.maximum;
        }
      }

      if (panel.configuration.setComponentParameter(panel.block_name, this.parameter.name, value))
        $(this).removeClass('default');
      else
        $(this).addClass('default');
    });

    this.configuration.addGlobalParameterListener(this.algorithm.name, parameter.name,
        this.block_name, function(previous_value, new_value) {
          if (input.value == previous_value)
            input.value = new_value;
        });
  }

  // Choices
  else
  {
    var select = document.createElement('select');
    select.className = 'form-control input-sm';
    cell.appendChild(select);

    select.block_name = this.block_name;
    select.parameter = parameter;

    var span       = document.createElement('span');
    span.className = 'help';
    span.innerHTML = '(' + parameter.type + ')';
    cell.appendChild(span);

    var value = parameter_values[parameter.name];

    for (var i = 0; i < parameter.choices.length; ++i)
    {
      var option   = document.createElement('option');
      option.text  = parameter.choices[i];
      option.value = parameter.choices[i];
      select.appendChild(option);

      if (option.value == value)
        select.selectedIndex = i;

      if (option.value == global_parameters[parameter.name])
      {
        $(option).addClass('default');

        if (option.value == value)
          $(select).addClass('default');
      }
    }

    $(select).change(function() {
      if (panel.configuration.setComponentParameter(panel.block_name, this.parameter.name, this.value))
        $(this).removeClass('default');
      else
        $(this).addClass('default');
    });

    this.configuration.addGlobalParameterListener(this.algorithm.name, parameter.name,
        this.block_name, function(previous_value, new_value) {
          var can_be_changed = (select.value == previous_value);

          for (var i = 0; i < select.options.length; ++i)
          {
            if (select.options[i].value == new_value)
            {
              $(select.options[i]).addClass('default');

              if (can_be_changed)
                select.selectedIndex = i;
            }
            else
            {
              select.options[i].className = '';
            }
          }

          if (!can_be_changed && (select.value == new_value))
            $(select).addClass('default');
        });
  }
}


//----------------------------------------------------------------------
// Add the controls to change a boolean parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype._createBoolControls = function(
    cell, parameter, parameter_values, global_parameters)
{
  var panel = this;

  var input       = document.createElement('input');
  input.className = 'form-control input-sm';
  input.type      = 'checkbox';
  cell.appendChild(input);

  input.block_name = this.block_name;
  input.parameter = parameter;

  input.checked = parameter_values[parameter.name];

  $(input).change(function() {
    panel.configuration.setComponentParameter(panel.block_name, this.parameter.name, this.checked);
  });


  this.configuration.addGlobalParameterListener(this.algorithm.name, parameter.name,
      this.block_name, function(previous_value, new_value) {
        if (input.checked == previous_value)
          input.checked = new_value;
      });
}


//----------------------------------------------------------------------
// Add the controls to change a string parameter value to the panel
//----------------------------------------------------------------------
beat.experiments.panels.BlockParameters.prototype._createStringControls = function(
    cell, parameter, parameter_values, global_parameters)
{
  var panel = this;

  var select = document.createElement('select');
  select.className = 'form-control input-sm';
  cell.appendChild(select);

  select.block_name = this.block_name;
  select.parameter = parameter;

  var span       = document.createElement('span');
  span.className = 'help';
  span.innerHTML = '(' + parameter.type + ')';
  cell.appendChild(span);

  var value = parameter_values[parameter.name];

  for (var i = 0; i < parameter.choices.length; ++i)
  {
    var option   = document.createElement('option');
    option.text  = parameter.choices[i];
    option.value = parameter.choices[i];
    select.appendChild(option);

    if (option.value == value)
      select.selectedIndex = i;

    if (option.value == global_parameters[parameter.name])
    {
      $(option).addClass('default');

      if (option.value == value)
        $(select).addClass('default');
    }
  }

  $(select).change(function() {
    if (panel.configuration.setComponentParameter(panel.block_name, this.parameter.name, this.value))
      $(this).removeClass('default');
    else
      $(this).addClass('default');
  });

  this.configuration.addGlobalParameterListener(this.algorithm.name, parameter.name,
      this.block_name, function(previous_value, new_value) {
        var can_be_changed = (select.value == previous_value);

        for (var i = 0; i < select.options.length; ++i)
        {
          if (select.options[i].value == new_value)
          {
            $(select.options[i]).addClass('default');

            if (can_be_changed)
              select.selectedIndex = i;
          }
          else
          {
            select.options[i].className = '';
          }
        }

        if (!can_be_changed && (select.value == new_value))
          $(select).addClass('default');
      });
}
