#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


"""Django Middleware to check for agreements to Legal Disclaimers and ToS"""

from django.contrib import messages
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

def remove_banner_message(request):
    '''Silences all banner messages'''

    storage = messages.get_messages(request)
    for message in storage:
        if message.extra_tags and 'banner' in message.extra_tags:
            message.extra_tags = message.extra_tags.replace('banner', '')
            message.extra_tags = message.extra_tags.strip()
            message.level = 0

def add_banner_message(request, text):
    '''Adds a new banner message if one is not already set'''

    storage = messages.get_messages(request)
    for message in storage:
        if message.extra_tags and 'banner' in message.extra_tags:
            message.message = text
            return

    messages.warning(request, text, extra_tags='banner', fail_silently=True)

class AgreementMiddleware(object):
    '''Checks for user agreement on legal disclaimers or terms-of-service.'''

    def process_request(self, request):

        if request.user.is_authenticated():
            if settings.TERMS_OF_SERVICE_VERSION > request.user.agreement.version:
                add_banner_message(request, mark_safe('We have modified our <a href="%s">Terms of Service (please take a moment to read them)</a>' % reverse('terms-of-service')))
            else:
                remove_banner_message(request)

        else:
            user_version = request.session.get('beat_disclaimer_version', 0)
            if settings.LEGAL_DISCLAIMER_VERSION > user_version:
                add_banner_message(request, mark_safe('By continuing to browse this website, you implicitly agree with our <a href="%s">Legal Disclaimer</a>' % reverse('legal-disclaimer')))
            else:
                remove_banner_message(request)
