#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
# encoding: utf-8

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


from django.core.management.base import BaseCommand, CommandError

from datetime import date

from ...models import Report

import sys
import random

class Command(BaseCommand):

    help = 'Cleanup outdated locked reports'

    def add_arguments(self, parser):
        parser.add_argument('--noinput', action='store_false', dest='interactive', default=True,
            help=('Tells Django to NOT prompt the user for input of any kind.'))


    def handle(self, *args, **options):
        clean = True

        if options['interactive']:
            try:
                answer = self.get_input_data('Make report(s) editable again with new unique id (Y/n)? ', 'y').lower()
            except KeyboardInterrupt:
                self.stderr.write("\nOperation canceled.")
                sys.exit(1)

            if answer != 'y':
                self.stdout.write('Cleanup canceled')
                sys.exit(1)

        if clean:
            expired_reports = Report.objects.filter(status=Report.LOCKED, expiration_date__lte=date.today())
            report_count = expired_reports.count()
            for expired_report in expired_reports:
                # Generate a unique report number
                used_numbers = map(lambda x: x.number, Report.objects.all())
                number = 0
                while (number == 0) or number in used_numbers:
                    number = random.randint(100000, 2**31)

                expired_report.status = Report.EDITABLE
                expired_report.number = number
                expired_report.expiration_date = None
                expired_report.save()

            self.stdout.write('{} locked report(s) successfully cleaned'.format(report_count))


    def get_input_data(self, message, default=None):
        """
        Override this method if you want to customize data inputs or
        validation exceptions.
        """
        raw_value = raw_input(message)

        if default and raw_value == '':
            raw_value = default

        return raw_value
