// general tests for the reports app
describe('reports app', function(){
	browser.ignoreSynchronization = true;

	// if there's an error in the web browser's console,
	// fail the test and print the error
	afterEach(function() {
		let util = require('util');
		browser.manage().logs().get('browser').then(function(logs) {
			// 'failed to load resource' is a familiar log error
			// when running adblockers. Don't fail on those.
			let failingLogs = logs
			.filter(l => !(/net::ERR_FAILED/.test(l.message)));

			expect(failingLogs.length).toEqual(0);

			if (failingLogs.length > 0) {
				console.log(`logs: ${util.inspect(failingLogs)}`);
			}
		});
	});

	// /reports
	describe('home', function(){
		beforeEach(function(){
			browser.get('http://localhost:8000/reports');
		});

		it('should load', function(){
			expect(browser.getTitle()).toEqual('BEAT - Public Reports');
		});

		it('should have no reports', function(){
			let noReportsText = browser.findElement(by.className('not-found'));
			expect(noReportsText.getText()).toBe('No report found');
		});
	});

	// /reports/user
	describe('home for the test user', function(){
		// login to the default user ('user') once before running all these tests
		beforeAll(function(){
			browser.get('http://localhost:8000/login/?next=/');
			//browser.findElement(by.partialLinkText('Sign-in')).click();
			browser.findElement(by.id('id_username')).sendKeys('user');
			browser.findElement(by.id('id_password')).sendKeys('user');
			browser.findElement(by.partialButtonText('Sign-in')).click();
			return browser.wait(function(){
				return browser.getCurrentUrl().then(function(url){
					const rxUserLoggedIn = /events\/user\//;
					return rxUserLoggedIn.test(url);
				});
			});
		});

		// go to user's reports page before each test
		beforeEach(function(){
			browser.get('http://localhost:8000/reports/user/');
		});

		// before adding a report, there shouldn't be any
		it('should have no reports', function(){
			let noReportsText = browser.findElement(by.className('not-found'));
			expect(noReportsText.getText()).toBe('No report found');
		});

		// create a report
		it('should create a new report called "test"', function(){
			let newReportButton = browser.findElement(by.partialLinkText('New'));
			newReportButton.click();

			// wait for dialog box to pop up
			browser.sleep(1000);

			let reportNameInput = browser.findElement(by.css('.has-error input'));
			reportNameInput.sendKeys('test');

			let submitButton = element(by.buttonText('Create'));
			expect(submitButton.isPresent()).toBeTruthy();
			submitButton.click();

			// wait for page to refresh
			browser.sleep(1000);

			let newReportLink = browser.findElement(by.linkText('user/test'));
			newReportLink.click();
			expect(browser.getTitle()).toBe('BEAT - Report');
		});
	});
});
