#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
# encoding: utf-8

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


import logging
logger = logging.getLogger(__name__)

from django.core.management.base import BaseCommand
from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.conf import settings
from django.contrib.sites.models import Site

from ...models import Leaderboard
from .... import __version__

import sys
import datetime

class Command(BaseCommand):

    help = 'Update and (optionally) notify owners on leaderboard changes'

    def handle(self, *ignored, **arguments):

        # Setup this command's logging level
        global logger
        arguments['verbosity'] = int(arguments['verbosity'])
        if arguments['verbosity'] >= 1:
            if arguments['verbosity'] == 1: logger.setLevel(logging.INFO)
            elif arguments['verbosity'] >= 2: logger.setLevel(logging.DEBUG)

        objects_updated = 0
        objects_changed = 0

        for obj in Leaderboard.objects.all():

            prev_date = obj.updated
            logger.info('Updating leaderboard for search %s...' % obj.search)
            changed = obj.update_experiments()
            objects_updated += 1

            emails = obj.notify.values_list('email', flat=True)

            # compare tables and notify if required
            if changed and emails:
                logger.info('Notifying interested parties for %s' % obj.search)
                objects_changed += 1
                current_site = Site.objects.get_current()
                template_path = 'search/leaderboard_changed.txt'
                subject = "Experiment ranking for leaderboard \"%s\" changed" % obj.search.fullname()
                mesg = EmailMessage(
                    subject.strip(),
                    render_to_string(template_path,
                      {
                        'leaderboard': obj,
                        'prev_date': prev_date,
                        'beat_version': __version__,
                        'site': current_site,
                        }
                      ),
                    settings.DEFAULT_FROM_EMAIL,
                    to=[],
                    bcc=emails,
                    )
                mesg.send()

            else:
                logger.debug('Leaderboard %s did not change or notification is off' % obj.search)

        logger.info('Leaderboard updates: %d / Notifications: %d' % \
            (objects_updated, objects_changed))
