#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.db import models
from django.dispatch import receiver

from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.conf import settings
from django.contrib.sites.models import Site

from .models import Leaderboard

@receiver(models.signals.pre_delete, sender=Leaderboard)
def notify_users(sender, instance, **kwargs):
    """Notify users the leaderboard was deleted"""
    emails = instance.notify.exclude(instance.author).values_list('email', flat=True)
    if emails:
        template_path = 'search/leaderboard_deleted.txt'
        subject = "Leaderboard \"%s\" was deleted" % instance.search.fullname()
        mesg = EmailMessage(
            subject.strip(),
            render_to_string(template_path,
              {
                'leaderboard': instance,
                'prev_date': prev_date,
                'beat_version': __version__,
                'site': current_site,
                }
              ),
            settings.DEFAULT_FROM_EMAIL,
            to=[settings.DEFAULT_FROM_EMAIL],
            bcc=emails,
            )
        mesg.send()
