#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################



from django import template
from django.conf import settings

register = template.Library()


#--------------------------------------------------


@register.inclusion_tag('team/panels/table.html', takes_context=True)
def team_table(context, objects, owner, id):
    '''Composes a team list table

    This panel primarily exists for user's team list page.

    Parameters:

        objects (iterable): An iterable containing Toolchain objects
        owner (bool): A flag indicating if the list is being created for the
          owner of his personal list page on the user-microsite or not.
        id: The HTML id to set on the generated table. This is handy for the
          filter functionality normally available on list pages.

    '''

    return {
            'request': context['request'],
            'objects': objects,
            'owner': owner,
            'panel_id': id,
            }


#--------------------------------------------------


@register.inclusion_tag('team/panels/actions.html', takes_context=True)
def team_actions(context, obj, display_count):
    '''Composes the action buttons for a particular team

    This panel primarily exists for showing action buttons for a given
    team taking into consideration it is being displayed for a given user.

    Parameters:

        obj (Team): The team object concerned for which the
          buttons will be drawn.

        display_count (bool): If true, displays the number of elements in the
          team in a button

    '''
    return {
            'request': context['request'],
            'display_count': display_count,
            'object': obj,
            }


#--------------------------------------------------


@register.inclusion_tag('ui/contribution_list.html')
def user_list(id):
    return { 'panel_id': id,
             'URL_PREFIX': settings.URL_PREFIX,
             'finder_placeholder': 'Find an User...',
             'owner': False,
           }


#--------------------------------------------------


@register.simple_tag(takes_context=True)
def total_shares_for_user(context, team):
    return team.total_shares(context['request'].user)
