/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 * 
 * This file is part of the beat.web module of the BEAT platform.
 * 
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 * 
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/* beat.ui.history.js

   Implementation of the 'contribution history' panel
*/


// Declaration of our namespaces
if (beat === undefined)
    var beat = {}

if (beat.ui === undefined)
    beat.ui = {}

if (beat.ui.history === undefined)
    beat.ui.history = {}



beat.ui.history.setup = function(panel_id, contribution_type, contribution_fullname,
                                 url_prefix, height, callback)
{
    $.get(url_prefix + '/api/' + contribution_type + '/' + contribution_fullname + '/?fields=history',
          function(data) {
              beat.ui.history.display(panel_id, data.history, contribution_type,
                                      contribution_fullname, url_prefix, height,
                                      callback);
          }
    );
}


beat.ui.history.display = function(panel_id, history, contribution_type, contribution_fullname,
                                   url_prefix, height, callback)
{
    function _nbVersions(from)
    {
        var infos = {
            nb_versions: 1,
            new_branches: [],
        };

        var parts = from.name.split('/');
        var ref_name = parts[0] + '/' + parts[1];

        for (var i = 0; i < from.next.length; ++i)
        {
            from.next[i].previous = from;

            var parts = from.next[i].name.split('/');
            var name = parts[0] + '/' + parts[1];

            if (name != ref_name)
                infos.new_branches.push(name);

            var infos2 = _nbVersions(from.next[i]);

            infos.nb_versions += infos2.nb_versions;
            infos.new_branches = infos.new_branches.concat(infos2.new_branches);
        }

        return infos;
    }

    var version_infos = _nbVersions(history);

    var parts = history.name.split('/');
    version_infos.new_branches = [parts[0] + '/' + parts[1]].concat(version_infos.new_branches);


    var container = $('#' + panel_id);
    container[0].style.height = '' + height + 'px';

    var ROW_HEIGHT         = 30;
    var BRANCH_COL_WIDTH   = 20;
    var GRAVATAR_COL_WIDTH = 30;

    var BRANCH_COLORS = [
        '#CC0000',
        '#00CC00',
        '#0000CC',
        '#cc007a',
        '#49cc28',
        '#00cc8e',
    ];

    var MONTH_ABBR = [
        'Jan',
        'Feb',
        'Mar',
        'Apr',
        'May',
        'Jun',
        'Jul',
        'Aug',
        'Sep',
        'Oct',
        'Nov',
        'Dec'
    ];


    var svg_height = Math.max(version_infos.nb_versions * ROW_HEIGHT, height);

    var paper = Raphael(panel_id, container.width()-40, svg_height);

    var month_col = paper.rect(0, 0, 50, svg_height);
    month_col.attr('fill', '#444444');
    month_col.attr('stroke', '#3333333');

    var month_col = paper.rect(50, 0, 40, svg_height);
    month_col.attr('fill', '#666666');
    month_col.attr('stroke', '#555555');

    var day_col = paper.rect(90, 0, 30, svg_height);
    day_col.attr('fill', '#888888');
    day_col.attr('stroke', '#777777');

    function _next(currents)
    {
        if (currents.length == 0)
            return null;

        var next = currents[0];
        var next_index = 0;

        for (var i = 1; i < currents.length; ++i)
        {
            var candidate = currents[i];
            if (candidate.creation_date < next.creation_date)
            {
                next = candidate;
                next_index = i;
            }
        }

        currents.splice(next_index, 1);

        if (next.next.length > 0)
            currents = currents.concat(next.next);

        return {
            next: next,
            currents: currents,
        };
    }

    var gravatars_x = (version_infos.new_branches.length + 1) * BRANCH_COL_WIDTH + 120;
    var names_x = gravatars_x + GRAVATAR_COL_WIDTH;

    var index = 1;
    var infos = _next([history]);

    while ((infos !== null) && (infos.next !== null))
    {
        var parts = infos.next.name.split('/');
        var branch = version_infos.new_branches.indexOf(parts[0] + '/' + parts[1]);

        infos.next.index = index;

        var color = BRANCH_COLORS[branch % BRANCH_COLORS.length];

        if (infos.next.previous)
        {
            var parts = infos.next.previous.name.split('/');
            var parent_branch = version_infos.new_branches.indexOf(parts[0] + '/' + parts[1]);

            if (parent_branch == branch)
            {
                var path = paper.path('M' + (120 + (version_infos.new_branches.length - branch) * BRANCH_COL_WIDTH) +
                                      ',' + (version_infos.nb_versions - index + 0.65) * ROW_HEIGHT + 'V' +
                                      (version_infos.nb_versions - infos.next.previous.index + 0.5) * ROW_HEIGHT);
            }
            else
            {
                var path = paper.path('M' + (120 + (version_infos.new_branches.length - branch) * BRANCH_COL_WIDTH) +
                                      ',' + (version_infos.nb_versions - index + 0.65) * ROW_HEIGHT + 'V' +
                                      (version_infos.nb_versions - infos.next.previous.index) * ROW_HEIGHT +
                                      'L' + (120 + (version_infos.new_branches.length - parent_branch) * BRANCH_COL_WIDTH) +
                                      ',' + (version_infos.nb_versions - infos.next.previous.index + 0.5) * ROW_HEIGHT);
            }

            path.attr('stroke', color);
        }

        infos = _next(infos.currents);
        index++;
    }


    var compare_link = paper.text(-8, 2, '[compare]');
    compare_link.attr('font', '12px Monaco,monospace');
    compare_link.attr('text-anchor', 'start');
    compare_link.attr('fill', '#005588');
    compare_link.attr('cursor', 'pointer');

    compare_link.hover(function() {
            this.attr('font-weight', 'bold');
        },
        function() {
            this.attr('font-weight', 'normal');
        });

    compare_link.click(function() {
        document.location = url_prefix + '/' + contribution_type + '/diff/' + this.contribution1 + '/' + this.contribution2 + '/';
    });

    compare_link.hide();


    index = 1;
    infos = _next([history]);
    var previous_date = new Date(infos.next.creation_date.replace(' ', 'T'));
    var older_contribution = true;

    while ((infos !== null) && (infos.next !== null))
    {
        var parts = infos.next.name.split('/');
        var branch = version_infos.new_branches.indexOf(parts[0] + '/' + parts[1]);

        var color = BRANCH_COLORS[branch % BRANCH_COLORS.length];

        var circle = paper.circle(120 + (version_infos.new_branches.length - branch) * BRANCH_COL_WIDTH,
                                  (version_infos.nb_versions - index + 0.5) * ROW_HEIGHT,
                                  3);
        circle.attr('fill', color);
        circle.attr('stroke', 'none');
        circle.attr('href', url_prefix + '/' + contribution_type + '/' + infos.next.name + '/');

        var rect = paper.rect(gravatars_x - 2,
                              (version_infos.nb_versions - index) * ROW_HEIGHT + (ROW_HEIGHT - 24) * 0.5 - 2,
                              28, 28);
        rect.attr('fill', color);
        rect.attr('stroke', 'none');
        rect.attr('href', url_prefix + '/' + contribution_type + '/' + parts[0] + '/');

        var image = paper.image(infos.next.author_gravatar + '?d=monsterid&r=pg&s=24',
                                gravatars_x, (version_infos.nb_versions - index) * ROW_HEIGHT + (ROW_HEIGHT - 24) * 0.5,
                                24, 24);
        image.attr('href', url_prefix + '/' + contribution_type + '/' + parts[0] + '/');

        var text = paper.text(names_x, (version_infos.nb_versions - index + 0.65) * 0.5*ROW_HEIGHT, infos.next.name);
        text.attr('font', '14px Monaco,monospace');
        text.attr('text-anchor', 'start');
        text.attr('href', url_prefix + '/' + contribution_type + '/' + infos.next.name + '/');

        if (infos.next.name == contribution_fullname)
        {
            text.attr('font-weight', 'bold');
            older_contribution = false;

            text.hover(function() {
                compare_link.hide();
            });
        }
        else if (!older_contribution)
        {
            text.contribution = infos.next.name;

            text.hover(function() {
                compare_link.show();

                compare_link.contribution1 = contribution_fullname;
                compare_link.contribution2 = this.contribution;

                var bbox = this.getBBox();
                compare_link.transform('T' + (bbox.x2 + 10) + ',' + bbox.cy);
            });
        }
        else
        {
            text.contribution = infos.next.name;

            text.hover(function(context) {
                compare_link.show();

                compare_link.contribution1 = this.contribution;
                compare_link.contribution2 = contribution_fullname;

                var bbox = this.getBBox();
                compare_link.transform('T' + (bbox.x2 + 10) + ',' + bbox.cy);
            });
        }

        if (index > 1)
        {
            var current_date = new Date(infos.next.creation_date.replace(' ', 'T'));

            if ((current_date.getYear() != previous_date.getYear()))
            {
                var text = paper.text(25, (version_infos.nb_versions - (index - 1) + 0.65) * 0.5*ROW_HEIGHT, previous_date.getFullYear());
                text.attr('font', '14px Monaco,monospace');
                text.attr('fill', '#ffffff');
            }

            if ((current_date.getYear() != previous_date.getYear()) || (current_date.getMonth() != previous_date.getMonth()))
            {
                var text = paper.text(70, (version_infos.nb_versions - (index - 1) + 0.65) * 0.5*ROW_HEIGHT, MONTH_ABBR[previous_date.getMonth()]);
                text.attr('font', '14px Monaco,monospace');
                text.attr('fill', '#ffffff');
            }

            if (current_date.getDate() != previous_date.getDate())
            {
                var text = paper.text(105, (version_infos.nb_versions - (index - 1) + 0.65) * 0.5*ROW_HEIGHT, previous_date.getDate());
                text.attr('font', '14px Monaco,monospace');
                text.attr('fill', '#ffffff');
            }

            previous_date = current_date;
        }

        infos = _next(infos.currents);
        index++;
    }

    var text = paper.text(25, 0.65 * 0.5*ROW_HEIGHT, previous_date.getFullYear());
    text.attr('font', '14px Monaco,monospace');
    text.attr('fill', '#ffffff');

    var text = paper.text(70, 0.65 * 0.5*ROW_HEIGHT, MONTH_ABBR[previous_date.getMonth()]);
    text.attr('font', '14px Monaco,monospace');
    text.attr('fill', '#ffffff');

    var text = paper.text(105, 0.65 * 0.5*ROW_HEIGHT, previous_date.getDate());
    text.attr('font', '14px Monaco,monospace');
    text.attr('fill', '#ffffff');

    if (callback)
        callback();
}
