/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 * 
 * This file is part of the beat.web module of the BEAT platform.
 * 
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 * 
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/* beat.ui.listselector.js

   Implementation of the list selector panel
*/


// Declaration of our namespaces
if (beat === undefined)
    var beat = {}

if (beat.ui === undefined)
    beat.ui = {}


/********************************* CLASS: ListSelector **********************************/

//----------------------------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------------------------
beat.ui.ListSelector = function(panel_id, modal)
{
    // Attributes
    this.id              = panel_id;
    this.modal           = modal;
    this.element         = $('#' + panel_id)[0];
    this.background      = null;
    this.entries         = null;
    this.selected_entry  = 0;

    // Callbacks
    this.onEntrySelected = null;
    this.onCancel = null;
}


//----------------------------------------------------------------------------------------
// Displays the panel with the provided list of entries
//----------------------------------------------------------------------------------------
beat.ui.ListSelector.prototype.display = function(entries, left, top, width)
{
    var panel = this;

    if (!panel.isHidden())
        panel.hide();

    this.update(entries);

    var width         = $(this.element).width();
    var height        = $(this.element).height();
    var window_width  = $(window).width();
    var window_height = $(window).height();

    if (left === undefined)
    {
        left = (window_width - width) / 2;
    }
    else if (this.modal)
    {
        left -= $(window).scrollLeft();

        if (left + width >= window_width)
            left = window_width - width - 4;
    }

    if (top === undefined)
    {
        top = (window_height - height) / 2;
    }
    else if (this.modal)
    {
        top -= $(window).scrollTop();

        if (top + height >= window_height)
            top = window_height - height - 4;
    }

    this.element.style.top  = '' + top + 'px';
    this.element.style.left = '' + left + 'px';

    this.background = document.createElement('div');
    this.background.className = 'ui-widget-overlay ui-front';
    $(this.background).insertBefore(this.element);

    if (!this.modal) {
        $(this.background).addClass('non_modal_back');
    }

    // Behavior
    $(this.background).click(function() {
        $(panel.element).hide();
        $(panel.background).remove();
        panel.background = null;

        if (panel.onCancel)
            panel.onCancel();

        $(window).off('keydown.list_selector');

        return false;
    });

    // Disable the context menu on the left click
    this.background.oncontextmenu = function (e) {
        e.preventDefault();
    };

    this.element.oncontextmenu = function (e) {
        e.preventDefault();
    };

    $(window).on('keydown.list_selector', function(event) {
        switch (event.which) {
            // Up arrow
            case 38 :
                {
                    var index = panel.selected_entry - 1;
                    while (index >= 0)
                    {
                        if (panel.entries[index].visible)
                            break;
                        --index;
                    }

                    if (index >= 0)
                    {
                        $(panel.entries[panel.selected_entry].row).removeClass('highlighted');
                        $(panel.entries[index].row).addClass('highlighted');
                        panel.selected_entry = index;
                    }
                    return false;
                }
                break;
            // Down arrow
            case 40 :
                {
                    var index = panel.selected_entry + 1;
                    while (index < panel.entries.length)
                    {
                        if (panel.entries[index].visible)
                            break;
                        ++index;
                    }

                    if (index < panel.entries.length)
                    {
                        $(panel.entries[panel.selected_entry].row).removeClass('highlighted');
                        $(panel.entries[index].row).addClass('highlighted');
                        panel.selected_entry = index;
                    }
                    return false;
                }
                break;
            // Enter key
            case 13 :
                {

                    $(panel.element).hide();

                    $(panel.background).remove();
                    panel.background = null;

                    if (panel.onEntrySelected)
                    {
                        panel.onEntrySelected(
                            panel.entries[panel.selected_entry].name,
                            panel.entries[panel.selected_entry].data);
                    }
                    $(window).off('keydown.list_selector');
                    return false;
                }
                break;
            // Escape key
            case 27 :
                {
                    panel.hide();
                    return false;
                }
                break;
        }
    });


    $(this.element).show();
}

//----------------------------------------------------------------------------------------
// Update the list of entries
//----------------------------------------------------------------------------------------
beat.ui.ListSelector.prototype.update = function(entries) {
    var panel = this;

    if (entries.length == 0)
    {
        this.hide();
        return;
    }

    if (this.entries)
    {
        old_entries = $('div.entries').find('li');
        old_selected = $(old_entries[panel.selected_entry]).find('span').text();
        $('div.entries').find('li').remove();

        panel.selected_entry = 0;
        for (var i = 0; i < entries.length; ++i)
        {
            if (entries[i].name === old_selected)
                panel.selected_entry = i;
        }
    } else {
        panel.selected_entry = 0;
    }

    this.entries = entries;

    var ul = $(this.element).find('div.entries ul');
    ul.empty();
    ul = ul[0];

    for (var i = 0; i < entries.length; ++i)
    {
        var entry = this.entries[i];
        entry.visible = true;

        entry.row = document.createElement('li');
        entry.row.index = i;
        ul.appendChild(entry.row);

        if (entry.color)
        {
            var span_color = document.createElement('span');
            span_color.className = 'color';
            span_color.style.backgroundColor = entry.color;
            entry.row.appendChild(span_color);
        }

        var span_name = document.createElement('span');
        $(span_name).html(entry.name);
        entry.row.appendChild(span_name);

        if (i == panel.selected_entry)
            entry.row.className = 'highlighted';

        $(entry.row).mouseenter(function() {
            $(panel.entries[panel.selected_entry].row).removeClass('highlighted');
            $(this).addClass('highlighted');
            panel.selected_entry = this.index;
            return false;
        });

        $(entry.row).click(function() {

            $(panel.element).hide();

            $(panel.background).remove();
            panel.background = null;

            if (panel.onEntrySelected)
            {
                panel.onEntrySelected(panel.entries[panel.selected_entry].name,
                                      panel.entries[panel.selected_entry].data);
            }

            $(window).off('keydown.list_selector');
            return false;
        });
    }
}

//----------------------------------------------------------------------------------------
// get selected label
//----------------------------------------------------------------------------------------
beat.ui.ListSelector.prototype.getSelected = function() {
    var panel = this;
    return panel.entries[panel.selected_entry];
}

//----------------------------------------------------------------------------------------
// Is panel hidden
//----------------------------------------------------------------------------------------
beat.ui.ListSelector.prototype.isHidden = function() {
    return this.background === null;
}

//----------------------------------------------------------------------------------------
// Hide the panel
//----------------------------------------------------------------------------------------
beat.ui.ListSelector.prototype.hide = function() {
    $(this.element).hide();
    $(this.background).remove();
    this.background = null;

    if (this.onCancel)
        this.onCancel();

    $(window).off('keydown.list_selector');
}
