#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.conf import settings
from django import template
from django.template.defaultfilters import stringfilter
import hashlib


register = template.Library()


#--------------------------------------------------


@stringfilter
def gravatar_hash(email):
    """ Computes the hash of an e-mail address to be recognized by gravatar.com
    """
    return hashlib.md5(email.lower()).hexdigest()


register.filter('gravatar_hash', gravatar_hash)


#--------------------------------------------------


@stringfilter
def gravatar_url(email):
    """ Returns the Gravatar URL of an e-mail address
    """
    if settings.USE_HTTPS_GRAVATAR:
        return 'https://secure.gravatar.com/avatar/' + hashlib.md5(email.lower()).hexdigest()
    else:
        return 'http://www.gravatar.com/avatar/' + hashlib.md5(email.lower()).hexdigest()


register.filter('gravatar_url', gravatar_url)
