#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.conf.urls import url
from django.conf import settings
from django.views.generic.base import TemplateView
from django.contrib.auth.views import logout

from . import views

from .registration.views import activate
from .registration.views import register
from .registration.forms import PreregistrationForm

urlpatterns = [

    url(r'^$',
        views.index,
        name='index',
        ),

    url(r'^login/$',
        views.login,
        name='login',
        ),

    url(r'^events/(?P<author_name>\w+)/$',
        views.activity_stream,
        name='activity-stream',
    ),

    url(r'^logout/$',
        logout,
        {'next_page': 'index'},
        name='logout',
        ),

    url(r'^docreq/(?P<author_name>\w+)/$',
        views.docreq,
        name='docreq',
        ),

    # Activation keys get matched by \w+ instead of the more specific
    # [a-fA-F0-9]{40} because a bad activation key should still get to the
    # view; that way it can return a sensible "invalid key" message instead
    # of a confusing 404.
    url(r'^signup/$', register,
        dict(success_url='registration-complete'),
        name='registration',
        ),

    url(r'^signup/complete/$',
        TemplateView.as_view(
          template_name=('registration/registration_pending.html' if \
              settings.PREREGISTRATION_ONLY else \
              'registration/registration_complete.html')
          ),
        name='registration-complete',
        ),

    url(r'^signup/activate/(?P<activation_key>\w+)/$',
        activate,
        name='registration-activation',
        ),

    url(r'^preregister/$',
        register,
        dict(
          success_url='pre-registration-complete',
          form_class= PreregistrationForm,
          template_name='registration/preregistration_form.html',
          ),
        name='pre-registration'
        ),

    url(r'^preregistration/complete/$',
        TemplateView.as_view(
          template_name='registration/preregistration_complete.html',
          ),
        name='pre-registration-complete',
        ),

]
